function SS = AltruismPolicyFct(S)  
% Gets policies and laws of motion for altruism model
%
% Input:
% Object S with required fields:
%    S(1).V.val:                Updated value-function guess 
%    S(1).options.UpWindDer:    optional. 1: Use upwind principle, 2: use
%                               centered derivatives.
%    S(1).options.Extrap:       optional. 1: Use FinElObj-built-in method
%                               (based on linear extrapolation of 2nd der.)
%                               2: Use "consumption-based" extrapolation
%    other fields:              See beginning of function where they are
%                               read out
%
% Output:
% Object S with updated fields:
%    S(1).cons:         consumption of player 1
%    S(1).consp:        consumption of player 2
%    S(1).g:            transfers by player 1 (to player 2)
%    S(1).gp:           transfers by player 2
%    S(1).RiskLov:      risk-taking of player 1 (=1 if she buys risky asset)
%    S(1).RiskLovp:     risk-taking of player 2 (=1 if he buys risky asset)
%    S(1).FU            flow utilities in all states (for both players)
%    S(i).a:            drift in dimensions i=1,2
%    S(i).s:            volatility in dimensions i=1,2
%    S(1).V.jac:        Jacobian
%    S(1).V.SecDer:     second derivatives of value function
%
% Calls:
% GetJac.m
%
% Called by:
% ValueFctIter.m
%
% Usage:
% S = AltruismPolicyFct(S);         % call directly
%   % or
% PolFctHdl = @AltruismPolicyFct;   % call using function handle
% S = feval(PolFctHdl,S);


if ~isfield(S(1),'options')         % read out upwind option if given
    UpWindDer = 0; 
    Extrap    = 1;
else
    if ~isfield(S(1).options,'UpWindDer')
        UpWindDer = 0;              % default is centered derivatives
    else
        UpWindDer = S(1).options.UpWindDer;
    end
    if ~isfield(S(1).options,'Extrap')
        Extrap    = 1;              % default is centered derivatives
    else
        Extrap    = S(1).options.UpWindDer;
    end
end

params = S(1).params;               % read out parameters
gamma = params.g;  r = params.r;  rho = params.rho;  u = params.u;
a = params.a; ap = params.ap;
SigmaNorm = params.SigmaNorm;  SigmaRisky = params.SigmaRisky;  


FullInd  = S(1).FullInd;            % read out full set of indices
nGridVec = S(1).nGridVec;

kk = S(1).xGrid;                    % read out grids for states
kkp= S(2).xGrid;
yy = S(3).xGrid;
yyp= S(4).xGrid;

if Extrap==1                        % check which extrapolation is desired:
    S = GetJac(S);                  % here: standard, built in GetJac.m
else                                % If special one is desired:
    S = GetJac(S,1:2,1:2,0);        % calculate derivative of value fct.
                                    % in first and second dimension for
                                    % both players without extrapolation.
    S = AltruismExtrapolate(S);     % Extrapolate value function with extra
end                                 % function.

                                    
%% Case where none is broke                                    
Vk = S(1).V.jac(FullInd{:},1,1);    % read out derivative of V in k-direction
Vkp= S(1).V.jac(FullInd{:},2,2);

if UpWindDer==1                     % Up-wind algorithm:
                                    % player 1's consumption decision:
    cUp      = Vk.^(-1/gamma);      % consumption when upward-looking.
    S(1).aUp = r*kk + yy - cUp;     % write into object
    ind1dn = FullInd;  ind1dn{1}=[1  1:nGridVec(1)-1];
    cDn      = cUp(ind1dn{:});      % get consumption when downward-looking
    S(1).aDn = r*kk + yy - cDn;     % and write into object.
    S(1).a   = GetUpWindLOM(S,1);   % Get resulting upwind consumption.
    c        = r*kk + yy - S(1).a;  % get consumption again
    S(1).cons = c;                  % write it into object 
                                    % player 2's consumption decision:
    cpUp     = Vkp.^(-1/gamma);
    S(2).aUp = r*kkp+ yyp- cpUp;    % analogous
    ind2dn   = FullInd;  ind2dn{2}=[1  1:nGridVec(2)-1];
    cpDn     = cpUp(ind2dn{:});
    S(2).aDn = r*kkp+ yyp- cpDn;
    S(2).a   = GetUpWindLOM(S,2);
    cp       = r*kkp+ yyp -S(2).a;
    S(1).consp = cp;

else                                % If centered derivatives are called for:

    S        = GetJacCnt(S);        % first get centered derivatives
                                    % player 1's consumption decision:
    Vkc      = S(1).V.JacCnt(FullInd{:},1,1);
    c        = Vkc .^(-1/gamma);    % Get consumption from this derivative
    S(1).a   = r*kk + yy - c;       % suggest this as law of motion
    S(1).cons = c;
                                    % player 2's consumption decision:
    Vkpc     = S(1).V.JacCnt(FullInd{:},2,2);
    cp       = Vkpc.^(-1/gamma);
    S(2).a   = r*kkp+ yyp- cp;
    S(1).consp = cp;

end


% Player 1's risk-taking decision:
RiskLov = AltruismRiskTaking(c,1,S);
sigma = SigmaNorm + RiskLov.*SigmaRisky;
S(1).s = sigma.*kk;                 % note that anyway risk at k0 is 0
S(1).RiskLov = RiskLov;

% Player 2's risk-taking decision.
RiskLovp = AltruismRiskTaking(cp,2,S);
sigmap = SigmaNorm + RiskLovp.*SigmaRisky;
S(2).s = sigmap.*kkp;	
S(1).RiskLovp = RiskLovp;


%% Start cases where one is broke
ind1Constr = FullInd;  ind1Constr{1} = 1;   % indices where pl.1 is constrained
ind2Constr = FullInd;  ind2Constr{2} = 1;   % indices where pl.2 is constrained
indBothConstr = FullInd; indBothConstr{1}=1; indBothConstr{2}=1; 
                                            % indices where both are
                                            % constrained.
                                            
c00 =  c(indBothConstr{:});     % read out "desired consumption" (=uncon-
cp00 =  cp(indBothConstr{:});   % strained) for points where both are broke
                                % before it's overwritten.

%% Cases where only one of the two is broke:                                
% grid points where he is broke:
c0   = c(ind2Constr{:});        % read out "desired consumption" here
cp0  = cp(ind2Constr{:});  yp0 = yyp(ind2Constr{:});
gdict= a^(1/gamma)*c0 - yp0;
gunc = max( 0, min( gdict, cp0-yp0 ) );
g(FullInd{:}) = 0;              % set up 0-array for transfers
g(ind2Constr{:}) = gunc;        % and fill with gunc where he's constrained
cp(ind2Constr{:}) = min( cp0, yp0+gunc );
                                % For him, correct consumption whenever his
                                % "desired" level is not feasible.
                                % (also fill point where both are broke:
                                % will be overwritten later)

                                % Now points where she has k=0:
cp0   = cp(ind1Constr{:});
c0   = c(ind1Constr{:});  y0 = yy(ind1Constr{:});
gpdict= ap^(1/gamma)*cp0 - y0;
gpunc = max( 0, min( gpdict, c0-y0 ) );
gp(FullInd{:}) = 0;
gp(ind1Constr{:}) = gpunc;
c(ind1Constr{:}) = min( c0, y0+gpunc);


%% Case where both are broke:
y00 = yy(indBothConstr{:}); yp00 = yyp(indBothConstr{:});
gunc00 = gunc(indBothConstr{:}); gpunc00 = gpunc(indBothConstr{:});
SheIncRich = (y00>yp00);

% First, do case where she's income rich:
SheConstr  = ( gunc00  + c00  > y00  );     % Sub-case where she's constrained
gstatdict  = (  a^(1/gamma)*y00  - yp00 ) ./ ( 1+ a^(1/gamma) );
gconstr    = max( 0, min( gstatdict , cp00-yp00 ) );
cconstr    = y00  - gconstr;
cpconstr   = yp00 + gconstr;
% Now, put together constrained and unconstrained cases:
g00SheRich = SheConstr.*gconstr  + (1-SheConstr).*gunc00;
c00SheRich = SheConstr.*cconstr  + (1-SheConstr).*c00   ;
cp00SheRich= SheConstr.*cpconstr + (1-SheConstr).*min(yp00+gunc00,cp00);

% Do case where he is income rich:
HeConstr   = ( gpunc00 + cp00 > yp00 );     % Sub-case where he is constrained
gpstatdict = ( ap^(1/gamma)*yp00 - y00  ) ./ ( 1+ap^(1/gamma) );
gpconstr   = max( 0, min( gpstatdict, c00 -y00  ) );
cpconstr   = yp00 - gpconstr;
cconstr    = y00  + gpconstr;
% Now, put together constrained and unconstrained cases:
gp00HeRich = HeConstr .*gpconstr + (1-HeConstr ).*gpunc00;
cp00HeRich = HeConstr .*cpconstr + (1-HeConstr ).*cp00   ;
c00HeRich  = HeConstr .* cconstr + (1-HeConstr ).*min(y00+gpunc00,c00);

% Now, put together cases where she's income rich and where he's inc.-rich:
g00        = SheIncRich     .* g00SheRich;  % transfers only when one is
gp00       = (1-SheIncRich) .* gp00HeRich;  % income rich
c00        = SheIncRich     .* c00SheRich   +  (1-SheIncRich).*c00HeRich ;
cp00       = SheIncRich     .* cp00SheRich  +  (1-SheIncRich).*cp00HeRich;

%% Collect results 
% Write back into arrays:

c(indBothConstr{:}) = c00;  cp(indBothConstr{:}) = cp00;
g(indBothConstr{:}) = g00;  gp(indBothConstr{:}) = gp00 ;

if any( abs( imag([c(:);cp(:)]) ) ) > 10^(-12) % If any consumption value
   display('Negative value of saving')         % is complex, give error
   error('Negative value of saving')
end

%   Write into object
S(1).cons = c; S(1).consp = cp; S(1).g = g; S(1).gp = gp;
S(1).a = r*kk + yy - c - g + gp;
S(2).a = r*kkp+ yyp- cp- gp+ g ;
S(1).FU(FullInd{:},1) = u(c)  + a *u(cp);
S(1).FU(FullInd{:},2) = u(cp) + ap*u(c) ;

SS = S;

