function z = AltruismRiskTaking(c,iPlr,S)
% This function gets risk-taking for player with index 'iPlr' (1 or 2),
% given consumption c (for player iPlr) which was obtained from upward
% derivatives.
%
% Inputs:
%   c:        nGrid1 x .. x nGridN array of upward-looking consumption for
%             player iPlr given current value function.
%   iPlr:     scalar, 1 or 2. Index for player whose risk-taking is to be
%             determined.
%   S:        object with the usual fields. Necessary:
%      kappa: Defines how close risk-taking is to true bang bang. 0: always 
%             50-50 portfolio. inf: true bang-bang. Smoothed logistically.
%
% Output:   
%   z;        nGrid1 x .. x nGridN array of numbers in (0,1): risky
%             portfolio share.
%
% Called by:
% AltruismPolicyFct.m

FullInd  = S(1).FullInd;                % read out fields from object
nGridi   = S(1).nGridVec(iPlr);
dx       = S(iPlr).dx;

jPlr = 3-iPlr;                          % index for other player
kki  = S(iPlr).xGrid;                   % player i's asset grid
kkj  = S(jPlr).xGrid;                   % other player's asset grid
iPoor = (kki<kkj);                      % determine where i is (asset-)
                                        % poorer than other player
if isfield(S(1).params,'kappa')
    kappa = S(1).params.kappa;          % set kappa-parameter if not given
else
    kappa = 1000;
end

ind1N1 = FullInd;  ind1N1{iPlr} = 1:nGridi-1;       % take derivative of 
dcdk    = diff(c,1,iPlr) ./  S(1).dxc(ind1N1{:});   % consumption to assets

ind1 = FullInd;  ind1{iPlr} = nGridi-1;             % extrapolate for top
ind2 = FullInd;  ind2{iPlr} = nGridi-2;             % grid point
Top  = dcdk(ind1{:}) + ( dcdk(ind1{:}) - dcdk(ind2{:}) ) ./ ...
       dx(ind2{:}) .* dx(ind1{:}) ;
dcdk = cat( iPlr, dcdk, Top );                      
                                        % If this is negative: risk-taker.
z = iPoor ./ (1 + exp( kappa*dcdk/S(1).params.rho ) ); % Smooth out with
                                        % logistic function. Normalization:
                                        % 1/rho is how consumption moves
                                        % in normal case with assets, kappa
                                        % governs steepness of reaction.
                                        