% This script solves a differential game between two altruistic agents who
% can consume, save and give transfers to each other. The horizon is finite.
% It uses ValueFctIter to solve the game. The optimal policies are not
% saved (to save time and memory), instead a movie is shown -- plots for 
% the movie are made by the function AltruismVisualizeVFI.m.


addpath('..','../SmallFct');    % need all functions for finite-element
                                % object on path: are in parent folder.
                                % Note: The
                                % directory 'AltruismUncer' must be the
                                % current directory.
clear S Saut1 Saut2 Swp options params;
                                % need to set up these objects anew
close all;                      % close all figures


%% Set up the object
% Options for value-function iteration algorithm:
epsConv   = 10^(-16);   % Should not stop here before T is reached.
ShowEvery = 100;        % message after every ... iterations
Howard    = 0;          % have to do Classical (i.e 0) option here
dtVar     = 1;          % have to set to 1 here: need variable time increment 
                        % close to T because consumption is very high!
dt        = 1;          % The last period: length equal to this value in years.
T         = 90;         % final year
maxIter   = 10*T/dt;    % number of iterations:  10*T/dt usually enough 
                        % to get >60 years together for dt=1.
pmmin     = 0;          % set p_middle equal to this value when dtVar=1.                        
Extend    = 0;          % 0: reflect back on upper k-grid boundary.
                        % 1: extrapolate v.f. there and jump out
UpWindDer = 0;          % 1: take upwind derivatives, 0: take centered der.
Extrap    = 0;          % 1: Extrapolate using GetJac.m, 2: Extrapolate
                        % using AltruismExtrapolate.m ("consumption-based")

% Grid properties:
N  = 60;                % Number of grid points

if Extend==0
   ptsExcl = round(0.25*N);% How many points to exclude (in plotting and for     
else                    % other results) when working with different 
   ptsExcl=2;           % algorithms (have to throw away more points when 
end                     % grid is not extended)
linGrid=1;

% Plotting parameters:
nArrows      = 10;      % set numbers of arrows in plots
ArrowScale   = 15;      % size of arrows
VisualizeVFI = 1;       % make movie: set to 1
S(1).VisualizeVFIFctHdl = @AltruismVisualizeVFI;
                        % tell object which code visualizes value-function
                        % iteration: AltruismVisualizeVFI.m

% Display size of grid, first put space line here                        
display(' ')            
display(sprintf('Grid size: N = %1.0f',N))
topK = 2500;            % usually, do topK = 2500 here

% Model parameters:
rho        = 0.034;     % rate of time preference   
r          = 0.03;      % interest rate
g          = 2;         % curvature of utility
u = @(c) c.^(1-g) / (1-g);  % set up function handle for utility
aa         = sqrt(0.3); % player 1's altruism coefficient
aap        = sqrt(0.2); % player 2's altruism coefficient
a = aa^g; ap = aap^g;   % obtain altruism parameters associated with these
yvec       = [25 50];   % 2 income levels for her
ypvec      = 25;        % only 1 income level for him
eta        = 0.1;       % hazard rate of income change
etap       = 0.1;
SigmaNorm  = 0.03;      % risk in savings: 0.01 is 1% standard deviation of return
SigmaRisky = SigmaNorm; % this is the risky asset
kappa      = 10;        % the higher this parameter, the closer portfolio
                        % is to bang-bang. At 0, always have 50-50
                        % portfolio. At 1, logistic smoothing.

% Assign function that calculates the optimal policies to object:
S(1).PolicyFctHdl = @AltruismPolicyFct;
% Assign function that obtains value-function guess
S(1).VGuessFctHdl = @AltruismVGuess;


% Collect options and assign to object:
options.Howard = Howard;
options.maxIter = maxIter;  options.epsConv = epsConv;
options.ShowEvery = ShowEvery;  options.dtVar = dtVar; 
options.dt = dt; options.pmmin = pmmin;  options.Extend = Extend;
options.UpWindDer = UpWindDer;  options.Extrap = Extrap;
options.VisualizeVFI = VisualizeVFI;
S(1).options = options;       % hand options over to S

% Collect parameters and hand over to object:
params.g = g;  params.r = r;  params.rho =  rho;  params.eta  = eta;
params.a = a;  params.ap=ap;  params.u = u;  params.kappa = kappa;
params.SigmaNorm  = SigmaNorm;  params.SigmaRisky = SigmaRisky;
S(1).params = params;         % hand over to S

% Create grid
if linGrid==1
    kgrid = (linspace(0,topK,N));     % build linear k-grid
    dk = topK / (N-1);
else
    Abs0 = -LogGridPar*yvec(1)/r;      % where logarithmic grid would converge to
    kgrid = GetLogGrid(0,topK,Abs0,N); % make logarithmic grid
end

% Put fundamental info into S.
S(1).nPlr    = 2;             % 2 players
S(1).GridVec = kgrid';        % Grid vectors for continuous dimensions
S(2).GridVec = kgrid';        % for second player
S(3).GridVec = yvec;          % Discrete dimension 3: player 1's income
N3 = length(yvec);
S(3).discr   = 1;             % Say that dimension is discrete
S(3).h   = [-eta eta; eta -eta];    % jump-hazard matrix
S(4).GridVec = ypvec;         % Dimension 4: player 2's income, similar
N4 = length(ypvec);
S(4).discr   = 1;
if N4==1
    S(4).h = 0;
else
   S(4).h = [-etap etap; etap -etap];
end

S = FinElObjSetUp(S);         % Set up the object

FullInd = S(1).FullInd;       % Read out full set of indices

kk = S(1).xGrid;              % Read out grids: N x N x N3 x N4.
kkp= S(2).xGrid;
yy = S(3).xGrid;
yyp= S(4).xGrid;


%% Initial guess for value function
VT = AltruismVFinalT(S);        % get value function in final period
S(1).V.val = VT;                % and put it into object

if Extrap==1
    S = GetJac(S);
else
    S = GetJac(S,1:2,1:2,0);        % Extrapolate value function
    S = AltruismExtrapolate(S);
end


%% Value-function iteration
S = ValueFctIter(S);               % do value-function iteration: This 
% automatically shows the evolution of policies. Don't save policies here
% to save time.

MG = 1:N-ptsExcl;                   % the main grid: exclude 10 or so uppermost
                                    % grid points for assets

%% Show vector for time and dt implied by iteration history
figure;
subplot(1,3,1), plot(S(1).dtHist), xlabel('iteration'), ylabel('dt')
                                    % plot history for time increments
subplot(1,3,2), plot(T-cumsum(S(1).dtHist)), xlabel('iterations'), ylabel('t')
title('Algorith summary')           % plot implied vector for time t
subplot(1,3,3), plot(S(1).distHist), xlabel('iteration'), ylabel('distance in v,v''')
                                    % plot distance of value function at
                                    % each iteration

                                    
%% Plot consumption
figure;                             % plot her and his consumption for low  
                                    % income state
subplot(1,2,1)
surf( kk(MG,MG,1,1), kkp(MG,MG,1,1), S(1).cons(MG,MG,1) );
xlabel('her k'), ylabel('his k'''), zlabel('c')
title(sprintf('Her consumption (y=%1.0f)',yvec(1)))

subplot(1,2,2)
surf(kk(MG,MG,1,1), kkp(MG,MG,1,1), S(1).consp(MG,MG,1));
xlabel('her k'), ylabel('his k'''), zlabel('c''')
title(sprintf('His consumption (y=%1.0f)',yvec(1)))


    

%% Plot transfers

figure;                             % figures for her transfers to him
for i=1:N3                          % loop over her income states
      subplot(N3,2,i)
      surf( kk(MG,MG,1,1), kkp(MG,MG,1,1), S(1).g(MG,MG,i) );
      xlabel('her w'), ylabel('his w'''), zlabel('g')
      title(sprintf('Her transfer g (y=%1.0f)',yvec(i)))
end

for i=1:N3                          % now plot his transfers to her
      subplot(N3,2,2+i)
      surf(kk(MG,MG,1,1), kkp(MG,MG,1,1), S(1).gp(MG,MG,i));
      xlabel('her w'), ylabel('his w'''), zlabel('g''')
      title(sprintf('His transfer g'' (y=%1.0f)',yvec(i)))
end



%% Check transfer motive

S = GetJac(S);

Vk  = S(1).V.jac(MG,MG,:,:,1,1);
Vkp = S(1).V.jac(MG,MG,:,:,1,2);
muProp = (Vkp-Vk)./Vk;
maxMuProp = max( muProp(:) );
if maxMuProp > 0                    % Give warning if positive transfer motive
    display('ACHTUNG, BABY!!!')
    figure;                         % plot transfer motives if problematic
    for i=1:N3
       for j=1:N4
          subplot(N3,N4,i+N4*(j-1))
          surf(kk(MG,MG,1,1), kkp(MG,MG,1,1), muProp(MG,MG,i,j));
          xlim([0 topK]), ylim([0 topK])
          title(sprintf('\\mu as propn of V_k: y=%1.0f, y''=%1.0f',[yvec(i) ypvec(j)]))
       end
    end
end
display(sprintf('Transfer motive reaches %1.1f percent of Vk',maxMuProp*100))




%% Risk-taking
% figure;                             % make plots where agent 1 takes risk
% for i=1:N3
%     subplot(N3,2,i)
%     pcolor(kk(MG,MG,1,1), kkp(MG,MG,1,1), 1*S(1).RiskLov(MG,MG,i,j));
%    % xlim([0 topK]), ylim([0 topK])
%     title(sprintf('Her risk taking (y=%1.0f)',yvec(i)))
%     subplot(N3,2,2+i)
%     pcolor(kk(MG,MG,1,1), kkp(MG,MG,1,1), 1*S(1).RiskLovp(MG,MG,i,j));
%    % xlim([0 topK]), ylim([0 topK])
%     title(sprintf('His risk taking (y=%1.0f)',yvec(i)))
% end
% 
% 

display('DONE!')