% This script solves a differential game between two altruistic agents who
% can consume, save and give transfers to each other. It uses ValueFctIter2
% to solve the game (i.e. value-function iteration with first a coarse and
% then a finer grid), calculates the stationary distribution using
% GetStatDensity and calculates the distribution of hitting times of the
% broke state using GetDistHitTimes. It also uses lots of other small
% functions to calculate and plot properties of the equilibrium.

addpath('..','../SmallFct','../Bewley');
                                % need all functions for finite-element
                                % object on path: are in parent folder.
                                % Also, need Bewley model. Note: The
                                % directory 'AltruismUncer' must be the
                                % current directory.

clear S Saut1 Saut2 Swp options params;
                                % need to set up these objects anew
close all;                      % close all figures

% options for extra calculations:
DoDensityMovie = 0;             % show movie of density evolution if 1
DoHitTimes     = 0;             % Calculate hitting-time distribution if 1
DoEquivVar     = 0;             % calculate consumption-equivalent-variation
                                % numbers

%% Set up the object
% Options for value-function iteration algorithm:
epsConv    = 10^(-5);   % Convergence criterion for value-function iteration
                        % (choose 10^(-5) for g=2, 10^(-8) for g=4,
                        % 10^(-11) for g=6; 10^(-4) for g=1.1,
maxIter    = 10000;     % Maximal iterations in value-function iteration
ShowEvery  = 1000;      % message after every ... iterations
Howard     = 0;         % Classical val-fct iteration: 0. Howard: 1. Hybrid: 2.
tUpdatePol = 2;         % How many periods to maintain policy in hybrid
                        % algorithm, i.e. for Howard=2.
dtVar      = 1;         % 1: time increment dt in value-function (vf)
                        % is variable, if 0...
dt         = 0.1;       % ... equal to this value.
pmmin      = 0;         % set p_middle equal to this value when dtVar=1.                        
Extend     = 0;         % 0: reflect back on upper k-grid boundary.
                        % 1: extrapolate v.f. there and jump out
UpWindDer  = 0;         % 1: take upwind derivatives, 0: take centered der.
Extrap     = 0;         % 1: Extrapolate using GetJac.m, 2: Extrapolate
                        % using AltruismExtrapolate.m ("consumption-based")

% Grid properties:
N  = 60;                % Number of grid points

if Extend==0
   ptsExcl = round(0.25*N);% How many points to exclude (in plotting and for     
else                    % other results) when working with different 
   ptsExcl=2;           % algorithms (have to throw away more points when 
end                     % grid is not extended)
MG = 1:N-ptsExcl;       % the main grid: exclude 10 or so uppermost
                        % grid points for assets

Nsm = 30;               % number of grid points to get guess
% Nsm = [20 40];        % can also do two pre-runs and build up grid slower
linGrid = 1;            % 1: Grid is linear. 0: Grid is log-type.
%LogGridPar = .7;        % the closer this is to 0, the closer to true log grid. 

% Plotting parameters:
nArrows      = 10;      % set numbers of arrows in plots
ArrowScale   = 7;       % size of arrows: try around here to get nice phase diagram
VisualizeVFI = 1;       % make movie of convergence if =1
S(1).VisualizeVFIFctHdl = @AltruismVisualizeVFI;
                        % tell object which code visualizes value-function
                        % iteration.

% Display size of grid, first put space line here:                       
display(' ')            
display(sprintf('Grid size: N = %1.0f',N))
topK = 2500;            % usually, do topK = 2500 here

% Model parameters:
rho        = 0.034;     % rate of time preference   
r          = 0.03;      % interest rate
g          = 2;         % curvature of utility
u = @(c) c.^(1-g) / (1-g);  % set up function handle for utility
aa         = sqrt(0.3); % player 1's altruism coefficient a
aap        = sqrt(0.2); % player 2's altruism coefficient a'
a = aa^g; ap = aap^g;   % obtain altruism parameters alpha, alpha' associated with these
yvec       = [25 50];   % Income levels: always choose two for player 1
ypvec= yvec;            % can have one or two for pl. 2
Yvec = cat(2,yvec,ypvec);
eta        = 0.1;       % hazard rate of income change
etap       = 0.1;
SigmaNorm  = 0.03;      % risk in savings: 0.01 is 1% standard deviation of return
SigmaRisky = 0.03;      % st.dev. of risky asset (set equal to that of safe
                        % asset if there is no portfolio choice; set 
                        % SigmaNorm=0 and SigmaRisky=0.03 for risk-taking 
                        % example from paper)
kappa      = 10;        % the higher this parameter, the closer portfolio
                        % is to bang-bang. At 0, always have 50-50
                        % portfolio. At 1, logistic smoothing.
                        
N3 = length(yvec);      % create hazard matrices for income for both 
h  =  [-eta eta; eta -eta];
N4 = length(ypvec);
if N4==1
    hp = 0;
else
    hp = [-etap etap; etap -etap];
end                        

% Assign function to object S that calculates the optimal policies:
S(1).PolicyFctHdl = @AltruismPolicyFct;
% Assign function that obtains value-function guess
S(1).VGuessFctHdl = @AltruismVGuess;


% Collect options and assign to object:
options.Howard = Howard;  options.tUpdatePol = tUpdatePol;
options.maxIter = maxIter;  options.epsConv = epsConv;
options.ShowEvery = ShowEvery;  options.dtVar = dtVar; 
options.dt = dt; options.pmmin = pmmin;  options.Extend = Extend;
options.UpWindDer = UpWindDer;  options.Extrap = Extrap;
options.VisualizeVFI = VisualizeVFI;
S(1).options = options;       % hand options over to S

% Collect parameters and hand over to object:
params.g = g;  params.r = r;  params.rho =  rho;  params.eta  = eta;
params.a = a;  params.ap=ap;  params.u = u;  params.kappa = kappa;
params.SigmaNorm  = SigmaNorm;  params.SigmaRisky = SigmaRisky;
S(1).params = params;         % hand over to S

% Create grid
if linGrid==1
    kgrid = (linspace(0,topK,N));      % build linear k-grid
    dk = topK / (N-1);
else
    Abs0 = -LogGridPar*yvec(1)/r;      % where logarithmic grid would converge to
    kgrid = GetLogGrid(0,topK,Abs0,N); % make logarithmic grid
end

% Put fundamental info into S.
S(1).nPlr    = 2;             % 2 players
S(1).GridVec = kgrid';        % Grid vectors for continuous dimensions for 1st...
S(2).GridVec = kgrid';        % ...and 2nd player
S(3).GridVec = yvec;          % Discrete dimension 3: player 1's income
S(3).discr   = 1;             % Say that dimension is discrete
S(3).h       = h;             % jump-hazard matrix
S(4).GridVec = ypvec;         % Dimension 4: player 2's income, similar
S(4).discr   = 1;
S(4).h       = hp;

S(1).Nsm = Nsm;               % grid size for smaller grid

S = FinElObjSetUp(S);         % Set up the object

FullInd = S(1).FullInd;       % Read out full set of indices

kk = S(1).xGrid;              % Read out grids: N x N x N3 x N4.
kkp= S(2).xGrid;
yy = S(3).xGrid;
yyp= S(4).xGrid;



%% Value-function iteration
S = ValueFctIter2(S);               % do value-function iteration



%% Get stationary density

if DoDensityMovie==1                % make movie of density iteration if     
    nold = zeros(N,N,N3,N4);        % asked for.
    nold(20,10,1,1) = 1;
    noldVec = nold(:);
    zmax = 0.02;
    TT = 100;
    NN = 1+ceil(TT/dt);
    PlotEvery = 10;

    FF = ( speye(S(1).nSts) + S(1).K*dt)'; % the forward matrix

    figure;
    subplot(2,1,1), surf(kgrid,kgrid,nold(:,:,1)),  zlim([0 zmax])
    subplot(2,1,2), surf(kgrid,kgrid,nold(:,:,2)),  zlim([0 zmax])
    set(gca,'nextplot','replacechildren');
    MM(1) = getframe;
    for i=2:NN                      % loop over time
        nnewVec = FF * noldVec;
        if mod(i,PlotEvery) == 0
            nold = reshape(noldVec,[N N N3*N4]);
            for j=1:N3*N4
                subplot(N3,N4,j), surf(kgrid,kgrid,nold(:,:,j)),  zlim([0 zmax])
            end
            title(sprintf('Evolution of density: t=%1.1f)',(i-1)*dt))
            MM(i) = getframe;
        end
        noldVec = nnewVec;
    end

end


nStat = GetStatDensity(S);          % obtain stationary density and plot
S(1).nStat = nStat;

nSP = N3*N4;                        % number of subplots
figure;                             % figure for densities
for i=1:N3
   for j=1:N4 
      subplot(N4,N3,j+N4*(i-1))
      surf(kk(MG(2:end),MG(2:end),1,1),kkp(MG(2:end),MG(2:end),1,1),nStat(MG(2:end),MG(2:end),i,j))
      xlim([0 topK]),  ylim([0 topK])
      xlabel('w'), ylabel('w'''), zlabel('n_{stat}')
      title(sprintf('Density: y=%1.0f, y''=%1.0f',[yvec(i) ypvec(j)]))
   end
end

%% plot marginal distribution over wealth (of both players)

nWealthMarg = sum( sum(nStat,4), 3);        % integrate out income dimensions to get marginal wealth distribution
nSheBroke = nWealthMarg(1,MG(2:end));       % mass strip: she is broke
nHeBroke  = nWealthMarg(MG(2:end),1);       % mass strip: he is broke
nBothBroke= nWealthMarg(1,1);               % mass point: both broke

plotRatioDens = 0.1;                        % size of density plot with respect to grid

figure;
contour(kk(MG(2:end),MG(2:end),1,1), kkp(MG(2:end),MG(2:end),1,1), ...
        nWealthMarg(MG(2:end),MG(2:end)), 0:0.0002:0.2 )
colormap('gray')
    
scaler = plotRatioDens * topK / max( [nSheBroke(:); nHeBroke] );

kLo = -1.1*topK*plotRatioDens;              % bounds for plotting 
kHi = 0.75*topK;
xlabel('her w'), ylabel('his w'''),
xlim([kLo kHi]), ylim([kLo kHi]), 


hold on,
plot([kLo kHi],[0 0],'-k',[0 0],[kLo kHi],'-k')     % plot axes at 0
kPlotVec = kk( MG(2:end), 1);
plot(kPlotVec, -scaler*nHeBroke,'-k')       % plot densities of mass strips 
plot(-scaler*nSheBroke,kPlotVec,'-k')       % when one of players is broke
plot(0,0,'ok')

fs = 14;    % set font size for % labels of mass regions
text(0.7*kHi, 0.5*kLo, sprintf('%1.1f%%',sum(nHeBroke  )*100),'VerticalAlignment','middle','FontSize',fs)
text(0.5*kLo, 0.7*kHi, sprintf('%1.1f%%',sum(nSheBroke )*100),'HorizontalAlignment','center','FontSize',fs)
text(0.5*kLo, 0.5*kLo, sprintf('%1.1f%%',sum(nBothBroke)*100),'HorizontalAlignment','center','VerticalAlignment','middle','FontSize',fs)


%% Calculate distortions

ckp = diff(S(1).cons ,1,2) / dk;    % derivatives of consumption with 
cpk = diff(S(1).consp,1,1) / dk;    % respect to other player's assets

uc  = S(1).cons .^(-g);             % marginal utility
ucp = S(1).consp.^(-g);

vkp = S(1).V.jac(:,1:N-1,:,:,1,2);  % derivative of value function with  
vpk = S(1).V.jac(1:N-1,:,:,:,2,1);  % respect to other player's wealth

asd  = ckp .* (vkp - a *ucp(:,1:N-1,:,:) ) ./  uc(:,1:N-1,:,:) ;
asdp = cpk .* (vpk - ap* uc(1:N-1,:,:,:) ) ./ ucp(1:N-1,:,:,:) ;
trr = (S(1).g > 0) | (S(1).gp > 0) ;
asd(trr(:,1:N-1,:,:)) = 0;  asdp(trr(1:N-1,:,:,:)) = 0;
                                    % altruistic-strategic distortion from
                                    % Euler equation (divided by marg.ut.)

% Could put A-function into the object!
% Another way to compute the distortion:
% A = @(f) reshape( S(1).K*f(:) , [N N N3 N4]);
% K = S(1).K;
% asd  = A(uc) ./ uc - (rho-r);
% asdp = A(ucp)./ucp - (rho-r);
% 
% % check if consumption growth rate is roughly A(u_c) / gamma.
% Acc = A( S(1).cons ) ./ S(1).cons ; % more or less fine, usually.
% surf( Acc(:,:,2) - (r-rho)/g  );  zlim([-.01 .02])

%lev = -.01:0.005:0.03;
lev = [0  0.001 0.003 0.005 0.007];

figure
colormap gray

subplot(1,2,1)
[C, h1] = contour( kk(MG,MG,1,1), kkp(MG,MG,1,1), asd(MG,MG,1), lev );
set(h1,'ShowText','on')
%pcolor( kk(MG,MG,1), kkp(MG,MG,1), asd(MG,MG,1) )  % can also do other
%                                                   %types of plots
%set(gca,'CLim',[-0.02 0.02]), colorbar('location','southoutside'),
%surf( kk(MG,MG,1,1), kkp(MG,MG,1,1), asd(MG,MG,1,1) ),  
xlim([0 kHi]), ylim([0 kHi]),zlim([-.001 0.01])
xlabel('w'), ylabel('w''')
title(sprintf('Altruistic-strategic distortions for her (y=%1.0f, y''=%1.0f)',[yvec(1) ypvec(1)]))

subplot(1,2,2)
[C2, h2] = contour( kk(MG,MG,1,1), kkp(MG,MG,1,1), asdp(MG,MG,1), lev );
set(h2,'ShowText','on')
xlim([0 kHi]), ylim([0 kHi]), zlim([-.001 0.01])
% surf( kk(MG,MG,1,1), kkp(MG,MG,1,1), asdp(MG,MG,1,1) ),  
% pcolor( kk(MG,MG,1,1), kkp(MG,MG,1,1), asdp(MG,MG,1) ), 
% set(gca,'CLim',[-0.02 0.02]), colorbar('location','southoutside'),
xlabel('w'), ylabel('w''')

title(sprintf('Altruistic-strategic distortions for him (y=%1.0f, y''=%1.0f)',[yvec(1) ypvec(1)]))

% The plots for high income are very similar...
%subplot(2,2,2)
%pcolor( kk(MG,MG,1,1), kkp(MG,MG,1,1), asd(MG,MG,2) ), %  zlim([-0.01 0.02])
%set(gca,'CLim',[-0.02 0.02]), colorbar %('location','southoutside'),
%xlabel('k'), ylabel('k''')
%surf( kk(MG,MG,1,1), kkp(MG,MG,1,1), asd(MG,MG,2) ),  zlim([-0.01 0.02])
%title('Altruistic-strategic distortion for her (high y)')

%subplot(2,2,4)
%surf( kk(MG,MG,1,1), kkp(MG,MG,1,1), asdp(MG,MG,2) ),  zlim([-0.01 0.02])
% contour( kk(MG,MG,1,1), kkp(MG,MG,1,1), asdp(MG,MG,2), lev )
%title('Altruistic-strategic distortion for him (high y)')


%% Plot consumption
i=1;
nRow = 1;  nCol = 2;                % size of plot
figure;                             % figure for player 1's consumption
colormap gray

%for i=1:N3
    subplot(nRow,nCol,1)
    surf( kk(MG,MG,1,1), kkp(MG,MG,1,1), S(1).cons(MG,MG,i,1) );
    xlim([0 topK]), ylim([0 topK]), axis tight
    xlabel('her w'), ylabel('his w'''), zlabel('c')
    title(sprintf('Her consumption (y=%1.0f, y''=%1.0f)',[yvec(i) ypvec(1)]))
        text(0,1500,80,'transfer region','color','white')
        text(1000,1000,100,'SS region','color','white')
        text(700,1500,80,'OC region','color','white')
%end

%for i=1:N3
    subplot(nRow,nCol,2)
    surf(kk(MG,MG,1,1), kkp(MG,MG,1,1), S(1).consp(MG,MG,i,1));
    xlim([0 topK]), ylim([0 topK]), axis tight
    xlabel('her w'), ylabel('his w'''), zlabel('c''')
    title(sprintf('His consumption (y=%1.0f, y''=%1.0f)',[yvec(i) ypvec(1)]))
        text(1500,0,80,'transfer region','color','white')
        text(1500,1500,100,'SS region','color','white')
        text(1500,700,80,'OC region','color','white')


%end

% Can also check consumption out of common K=k+k' here:
% 
% K  = kk(MG,MG,1) + kkp(MG,MG,1);
% C  = S(1).cons(MG,MG,1)  ./ K;
% Cp = S(1).consp(MG,MG,1) ./ K;
% 
% % Now, consumption out of common K to compare to P-K-space
% 
% subplot(nRow,nCol,3)
%     surf(kk(MG,MG,1,1), kkp(MG,MG,1,1), C );
%     xlim([0 topK]), ylim([0 topK]), zlim([0 rho])
%     xlabel('her k'), ylabel('his k'''), zlabel('C')
%     title(sprintf('Out of K (y=%1.0f)',yvec(1)))
% subplot(nRow,nCol,4)
%     surf(kk(MG,MG,1,1), kkp(MG,MG,1,1), Cp );
%     xlim([0 topK]), ylim([0 topK]), zlim([0,rho])
%     xlabel('her k'), ylabel('his k'''), zlabel('C''')
%     title(sprintf('Out of K (y=%1.0f)',yvec(1)))
    
    

%% Plot transfers

figure;

MGg = 1:floor(0.9*N);             % plot more w-levels for transfers

gmax = [S(1).g(MGg,MGg,:,:); S(1).gp(MGg,MGg,:,:)];
gmax = max( gmax(:) );              % get max for z-axis
gmax = ceil( gmax/5 )*5;

el   = 5;                           % view: elevation angle in degrees
turn = 5;                           % turn angle in view

colormap gray

subplot(2,2,1)                      % her transfer when both poor
surf(kk(MGg,MGg,1,1), kkp(MGg,MGg,1,1), S(1).g(MGg,MGg,1,1));
xlabel('her w'), ylabel('his w'''), zlabel('g'),
axis tight, zlim([0 gmax])
view([-turn, el])                   % set angle of view
title(sprintf('Her transfer (y=%1.0f, y''=%1.0f)',[yvec(1) ypvec(1)]))

subplot(2,2,3)                      % her transfer when she rich, he poor
surf(kk(MGg,MGg,1,1), kkp(MGg,MGg,1,1), S(1).g(MGg,MGg,2,1));
xlabel('her w'), ylabel('his w'''), zlabel('g'),
axis tight,  zlim([0 gmax])
view([-turn, el])
title(sprintf('Her transfer (y=%1.0f, y''=%1.0f)',[yvec(2) ypvec(1)]))

subplot(2,2,2)                      % His transfer when both poor
surf(kk(MGg,MGg,1,1), kkp(MGg,MGg,1,1), S(1).gp(MGg,MGg,1,1));
xlabel('her w'), ylabel('his w'''), zlabel('g'''),
axis tight,  zlim([0 gmax])
view([-90+turn, el])
title(sprintf('His transfer (y=%1.0f, y''=%1.0f)',[yvec(1) ypvec(1)]))

subplot(2,2,4)
if N4>1             % if his income also varies: plot her 
   surf(kk(MGg,MGg,1,1), kkp(MGg,MGg,1,1), S(1).g(MGg,MGg,2,2));
   zlabel('g'), axis tight,  zlim([0 gmax]), view([-turn, el])
   title(sprintf('Her transfer (y=%1.f, y''=%1.0f)',[yvec(2) ypvec(2)]))
else                % if his income does not vary, plot his transfer when she's rich:
    surf(kk(MGg,MGg,1,1), kkp(MGg,MGg,1,1), S(1).gp(MGg,MGg,2,1));
    zlabel('g'''), axis tight,  zlim([0 gmax]), view([-90+turn, el])
    title(sprintf('His transfer (y=%1.0f, y''=%1.0f)',[yvec(2) ypvec(1)]))
end
xlabel('her w'), ylabel('his w'''), 
 

%% Check transfer motive

S = GetJac(S);

Vk  = S(1).V.jac(MG,MG,:,:,1,1);
Vkp = S(1).V.jac(MG,MG,:,:,1,2);
muProp = (Vkp-Vk)./Vk;
maxMuProp = max( muProp(:) );
if maxMuProp > 0                    % Give warning for positive transfer motive
    display('ACHTUNG, BABY!!!')
    figure;                         % plot transfer motives if problematic
    for i=1:N3
       for j=1:N4
          subplot(N3,N4,i+N4*(j-1))
          surf(kk(MG,MG,1,1), kkp(MG,MG,1,1), muProp(MG,MG,i,j));
          xlim([0 topK]), ylim([0 topK])
          title(sprintf('\\mu as propn of V_k: y=%1.0f, y''=%1.0f',[yvec(i) ypvec(j)]))
       end
    end
end
display(sprintf('Transfer motive reaches %1.1f percent of Vk',maxMuProp*100))




%% Marginal densities

S = GetMargDensity(S);      % Get marginal densities for all dimensions
figure;                     % and plot them
for i=1:4
    subplot(2,2,i)
    if S(i).discr==1
        bar(S(i).GridVec,S(i).nMarg);
    else
        plot(S(i).GridVec,S(i).nMarg );
            xlim([0 topK])
    end
    title(sprintf('Marginal density of dimension %1.0f',i))
end


%% Simulate and plot history
% x0 = [5; 1200; 1; 1];               % initial state for history
% dth= 0.1;                           % time increment for history 
% Th = 40;                            % length of history
% tChange = 20;                       % time at which income goes up for her
% Nh = ceil(Th/dth);                  % number of indeces for history
% indChange = round(Nh*tChange/Th+1); % index at which income changes
% shocks = ones(4,Nh);                % set up shock histories
% shocks(1:2,:) = 0;                  % no shocks to capital
% shocks(3,indChange:end) = 2;        % set high income for her after tChange
% 
% history = GetHistory(S,x0,dth,Th,shocks);
% Problem here: Her consumption is discontinuous, but GetHistory does not
% allow for this!

%% Risk-taking: Plots
figure;                               % make plots where agent 1 takes risk
colormap(gray)
MGr = MGg;                            % plot on a larger grid
lev = [0.5 0.5];                      % only plot one level line
if SigmaRisky>SigmaNorm
    strTitle = 'risk-taking';         % different title for figure if there
else                                  % is a portfolio choice
    strTitle = 'risk-lovingness';
end

subplot(1,2,1)                        % only plot for high income
contourf(kk(MGr,MGr,1,1), kkp(MGr,MGr,1,1), 1*S(1).RiskLov(MGr,MGr,end,end),lev);
title([ 'Her ', strTitle, sprintf(' (y=%1.0f y''=%1.0f)',yvec(end),ypvec(end)) ])
xlabel('w'), ylabel('w''')

subplot(1,2,2)
contourf(kk(MGr,MGr,1,1), kkp(MGr,MGr,1,1), 1*S(1).RiskLovp(MGr,MGr,2,2), lev);
title([ 'His ', strTitle, sprintf(' (y=%1.0f y''=%1.0f)',yvec(end),ypvec(end)) ])
xlabel('w'), ylabel('w''')



%% Get moments of stationary distribution

S = GetMoments(S);

% Get aggregate capital stock and GDP, show this.
Cpl = 0.5*(S(1).moments.m(1)+S(1).moments.m(2));
GDP = 0.5*( mean(yvec) + mean(ypvec) ) + r*Cpl;
display(sprintf('K = %1.2f, GDP = %1.2f',[Cpl GDP] ))

gg = S(1).g;                    % vectorized transfers
gFlows = gg(:)>0;               % where transfers flow
gtime = sum( nStat(gFlows)  );  % percentage of time transfer>0
display(sprintf('Plr. 1 gives transfers %1.1f percent of time',100*gtime))
gTot = sum( gg(:).*nStat(:) / gtime );  % Average transfer (conditional on >0)
display(sprintf('Average transfer from plr. 1: %1.1f',gTot))

heBroke = (kkp==0);
heBrokeTime = sum( nStat(heBroke) );
display(sprintf('Plr. 2 broke %1.1f percent of time',heBrokeTime*100));

ggp = S(1).gp;                  % vectorized transfers
gpFlows = ggp(:)>0;               % where transfers flow
gptime = sum( nStat(gpFlows)  );  % percentage of time transfer>0
display(sprintf('Plr. 2 gives transfers %1.1f percent of time',100*gptime))
gpTot = sum( ggp(:).*nStat(:) / gptime );  % Average transfer (conditional on >0)
display(sprintf('Average transfer from plr. 2: %1.1f',gpTot))

sheBroke = (kk==0);
sheBrokeTime = sum( nStat(sheBroke) );
display(sprintf('Plr. 1 broke %1.1f percent of time',sheBrokeTime*100));




%% Calculate transfer-wealth derivative

clim = 2*S(1).cons(2,:,:) - S(1).cons(3,:,:); % linearly extrapolate consumption
                                    % function from 2nd grid point downward
                                    % to get limit consumption
cplim= 2*S(1).consp(:,2,:)-S(1).consp(:,3,:);
                                    
HerPartyJump = clim - S(1).cons(1,:,:) ;    % calculate the jump in consumption when going
HisPartyJump = cplim-S(1).consp(:,1,:) ;    % broke

HerTrDer = -1 ./ ( 1 + HerPartyJump./S(1).gp(1,:,:) ) ; % calculate the transfer derivative
HerTrDer( S(1).gp(1,:,:)==0 ) = NaN;                    % get rid off it where transfers don't
HisTrDer = -1 ./ ( 1 + HisPartyJump./S(1).g(:,1,:) ) ;  % flow.
HisTrDer( S(1).g(:,1,:) ==0 ) = NaN;

figure;                             % plot transfer derivative (for his transfer
% in region where we can be confident: below 1300, consumption ceases to be
% linear and we can't trust in extrapolation to get clim. Above 2500, we
% see the effects of getting close to the end of the grid (this is for the
% baseline numerical experiment with N=1,000 and topK=3,000. Parameters:
% alpha=0.3, alpha'=0.1, gamma=2, sigma=0.03, r=0.03, rho=0.034, y=[20,30],
% eta=0.1, yp=25.
% subplot(1,2,1); 
plot( kgrid, HerTrDer(1,:,1), kgrid, HerTrDer(1,:,2))
title('Her transfer-wealth derivative'), xlabel('w'), ylabel('dg''/dw'), ylim([-1 0])
%xlim([1300 2500])
% subplot(1,2,2); 
% plot( kgrid, HisTrDer(:,1,1), kgrid, HisTrDer(:,1,2) )
% title('Transfer derivative'), xlabel('w'''), ylabel('dg''/dw'), ylim([-1 0])
% xlim([1300 2500])
% We see that the transfer derivative is roughly equal to -0.5 and slightly
% increasing in w'.


%% Calculate transfer-income derivatives (only valid if both players have exactly 2 income states

if N4>1                     % If both agents have income risk, can calculate transfer-income derivative
    BothGpPos = (S(1).gp(1,:,1,2)>0) & (S(1).gp(1,:,2,1)>0);   % Get points where he 
                            % gives transfers in both y=25,y'=50 and 
                            % y=50, y'=25 situation
    dGpdY = squeeze( S(1).gp(1,:,2,1) - S(1).gp(1,:,1,1) ) / diff(yvec);
                            % by how much does g' change if her income
                            % changes?
    dGpdYp= squeeze( S(1).gp(1,:,1,2) - S(1).gp(1,:,1,1) ) / diff(ypvec);
                            % by how much does g' change if his income
                            % changes?
    HerTrIncDer = dGpdY - dGpdYp;   % by how much does g' change if his
    HerTrIncDer(~BothGpPos) = NaN;  % income goes to her?
    
    BothGPos  = (S(1).g(:,1,2,1)>0) &  (S(1).g(:,1,1,2)>0) ;
    dGdYp = squeeze( S(1).g(:,1,1,2) - S(1).g(:,1,1,1) ) / diff(ypvec);
    dGdY  = squeeze( S(1).g(:,1,2,1) - S(1).g(:,1,1,1) ) / diff(yvec);
    HisTrIncDer = dGdYp - dGdY;     % by how much does g change if her income
    HisTrIncDer(~BothGPos)  = NaN;  % goes to him?
    
    figure;                         % plot the two TIDs

    subplot(2,1,1), plot(kgrid,HerTrIncDer)
    title('His transfer-income derivative')
    xlabel('w'''), ylabel('dg''/dy - dg''/dy'''), %ylim([-1.1, 0])

    subplot(2,1,2), plot(kgrid,HisTrIncDer)
    title('Her transfer-income derivative')
    xlabel('w'),   ylabel('dg/dy'' - dg/dy'),     %ylim([-1.1,0])
    

end




%% Get autarkic policies (needed for phase diagram and efficiency assessment)
clear Saut1 Saut2 Swp                          % set up 3 Bewley models: the following
paramsAut.g = g;  paramsAut.r = r;  paramsAut.rho =  rho;  % are the same for all
paramsAut.u = u;  paramsAut.sigma = SigmaNorm;        % three.
Saut1(1).params = paramsAut;   
Saut1(1).options = options;  
Saut1(1).options.ShowEvery = 2*maxIter; % don't show progress
Saut1(1).options.VisualizeVFI = 0;     % no movie here
Saut1(1).PolicyFctHdl = @BewleyPolicyFct;
Saut1(1).GridVec = kgrid';

Saut1(2).discr = 1;

Saut2 = Saut1;                          % copy the joint stuff for all
Swp   = Saut1;

Saut1(1).s = SigmaNorm*[kgrid' kgrid'];
Saut1(2).GridVec = yvec;                % autarkic problem for first agent
Saut1(2).h     = h;



Saut2(1).s = SigmaNorm*[kgrid' kgrid'];
Saut2(2).GridVec = ypvec;               % autarkic problem for second agent
Saut2(2).h = hp;
if N4>1
    Saut2(1).s = SigmaNorm*[kgrid' kgrid'];
else
    Saut2(1).s = SigmaNorm*kgrid';
end

wpKGrid = linspace(0,2*topK,2*N-1)';
Swp(1).GridVec = wpKGrid; % make grid twice as large for k

if N4>1
    Swp(2).GridVec = [yvec(1)+ypvec(1), yvec(1)+ypvec(2), ...
                      yvec(2)+ypvec(1), yvec(2)+ypvec(2)];
    Swp(2).h = [-eta-etap,  etap    ,  eta    ,  0   ;
                 etap    , -eta-etap,  0      , eta  ;
                 eta     ,  0       ,-eta-etap, etap ;
                 0       , eta      , etap    ,-eta-etap];
    Swp(1).s = SigmaNorm/sqrt(2)*[wpKGrid, wpKGrid, wpKGrid, wpKGrid];      % almost doesn't matter here if we let
else                                                                        % planner diversify portfolio (divide by sqrt(2))
    Swp(2).GridVec = yvec+ypvec;            % in the pooled problem         % or not
    Swp(2).h = h;
    Swp(1).s = SigmaNorm/sqrt(2)*[wpKGrid  wpKGrid];
end

display('Run autarkic models:')
%display('Bewley model for agent 1:')    % solve all models
Saut1 = GetBewleyResults(Saut1);
ConsAut1 = Saut1(1).cons;  aAut1 = Saut1(1).a;  clear Saut1;
%display('Bewley model for agent 2:')
Saut2 = GetBewleyResults(Saut2);
ConsAut2 = Saut2(1).cons;  aAut2 = Saut2(1).a;  clear Saut2;
%display('Bewley model for joint problem 2:')
Swp = GetBewleyResults(Swp);
ConsWp = Swp(1).cons;  aWp = Swp(1).a;  VWp = Swp(1).V.val;

aAut1 = aAut1(:,:,ones(1,N),ones(1,N4));     % Extend 
aAut1 = permute(aAut1,[1 3 2 4]);

aAut2 = aAut2(:,:,ones(1,N),ones(1,N3));
aAut2 = permute(aAut2,[3 1 4 2]);


%% Make arrow diagrams (a phase diagram, but there is a shock)
if N>nArrows
    PlotEvery = floor(N/nArrows);   % want 12 arrows per axis, roughly...
else
    PlotEvery   = 1;
end
if Extend==0
    PlotInd = 1:PlotEvery:N;
else
    PlotInd = 1:PlotEvery:N-10;
end
KK = kgrid(PlotInd(end));
dd = 0.05*kgrid(PlotInd(end));

figure;
for i=1:N3                          % loop over her income levels
    for j=1:N4                      % loop over his income levels
    subplot(N3,N4,N3*(i-1)+j)
    quiver(kk(PlotInd,PlotInd,1,1),kkp(PlotInd,PlotInd,1,1), ...
        ArrowScale*S(1).a(PlotInd,PlotInd,i,j),ArrowScale*S(2).a(PlotInd,PlotInd,i,j),0, ...
       'Color','b','linewidth',1)
    hold on,
    quiver(kk(PlotInd,PlotInd,1,1),kkp(PlotInd,PlotInd,1,1), ...
         ArrowScale*aAut1(PlotInd,PlotInd,i,j), ArrowScale*aAut2(PlotInd,PlotInd,i,j),0, ...
         'Color','r','linewidth',1)  % need to adjust scale by hand here: 
                                     % could not figure out how to set arrow
                                     % length. Factor 0.15 works
    xlim([kLo kHi]), ylim([kLo kHi])                                 
    if i==2
        xlabel('w')
    end
    if j==1
       ylabel('w''') 
    end
    title(sprintf('y=%1.0f, y''=%1.0f',[yvec(i) ypvec(j)]))
    end
end

text(.5*KK,kgrid(PlotInd(end)-1),'red: autarkic','Color','r','VerticalAlignment','top')
text(.5*KK,kgrid(PlotInd(end)-3),'blue: altruism','Color','b','VerticalAlignment','top')

%% Quantify inefficiencies (gamma*, cons.equiv.var., and eta*)
if DoEquivVar==1
Heta = @(eta) (  ( a.*eta+(1-eta) )  ./  ( eta+(1-eta).*ap )  ).^(1/g);
nSts = S(1).nSts;
vv = reshape( S(1).V.val(FullInd{:},:) , [nSts 2] );
gSymm = zeros( S(1).nGridVec' );   etaSymm = zeros( S(1).nGridVec' );
ConsBewleyMat = zeros(N,N,N3,N4);            % set up matrix to store Bewley
                                       % consumption plan on k/k' grid
display('Compute equivalent variation (inefficiencies)...')                                       

for i=1:nSts
    ki = kk(i);          kpi = kkp(i);          % read out indices of grid
    yind = S(3).ind(i);  ypind = S(4).ind(i);   % points
    Kiind = round(1+(ki+kpi)/dk);               % get index for joint K in WP-model
    if N4>1                                     % get index for joint Y in WP-model
        Yiind = (yind-1)*2 + ypind;
    else
        Yiind = yind; 
    end
    ConsBewleyMat(i) = ConsWp(Kiind,Yiind);     % read out consumption and value in
    VBewley = VWp(Kiind,Yiind);                 % WP-model
    veqm  = vv(i,1);  vpeqm = vv(i,2); %  VBewley = 
    geta  = @(eta) ( (1 +a*Heta(eta).^(1-g)).*VBewley ./ veqm  ).^(1/(1-g)) ...
                   ./ ( 1+Heta(eta) )    -    1 ;
    gpeta = @(eta) ( (ap+  Heta(eta).^(1-g)).*VBewley ./ vpeqm ).^(1/(1-g)) ...
                   ./ ( 1+Heta(eta) )    -    1 ;
    gSymmCheck = @(eta) geta(eta) - gpeta(eta);
    etaSymm(i) = fzero(gSymmCheck,[0 1]);
    gSymm(i)   = geta( etaSymm(i) );
end

figure
colormap gray

subplot(1,2,1)                  % plot inefficiencies in contours 
lev = [0.008 0.01 0.012 0.014 0.02 0.03];
[C1,h1] = contour( kk(MG,MG,1,1), kkp(MG,MG,1,1), gSymm(MG,MG,1,1) , lev);
set(h1,'ShowText','on')
%pcolor( kk(MG,MG,1,1), kkp(MG,MG,1,1), gSymm(MG,MG,2,2) ),
%set(gca,'CLim',[0 .04]), colorbar('location','southoutside'),
xlabel('w'), ylabel('w'''),
title(sprintf('\\gamma^* (y=%1.0f, y''=%1.0f)',[yvec(1) ypvec(1)]))

AA = nStat(MG,MG,:,:) .* gSymm(MG,MG,:,:);  % calculate average inefficiency,
AvgIneff = sum( AA(:) );                    % weighted by ergodic dist.
display(sprintf('Avg. inefficiency (ergodic weights): %1.2f%%',100*AvgIneff))

AA = nStat(MG,MG,:,:) .*  asd(MG,MG,:,:);   AvgAsd  = sum(AA(:));   % show average distortion
AA = nStat(MG,MG,:,:) .* asdp(MG,MG,:,:);   AvgAsdp = sum(AA(:));
display(sprintf('Avg. EE distortion: %1.2f%% (pl.1), %1.2f%% (pl.2)',100*[AvgAsd AvgAsdp]))

subplot(1,2,2)
%pcolor( kk(MG,MG,1), kkp(MG,MG,1), etaSymm(MG,MG,1) )
%set(gca,'CLim',[0 1]), colorbar('location','southoutside'), 
lev = 0:0.1:1;
[C2,h2]=contour( kk(MG,MG,1,1), kkp(MG,MG,1,1), etaSymm(MG,MG,1,1), lev);
set(h2,'ShowText','on')
xlabel('w'), ylabel('w'''),
title(sprintf('\\eta^* (y=%1.0f, y''=%1.0f)',[yvec(1) ypvec(1)]))

% subplot(2,2,3)        % These look exactly like low-income ones
% surf( kk(:,:,1), kkp(:,:,1), gSymm(:,:,2) ), title('g_{symm} (high y)')
% subplot(2,2,4)
% surf( kk(:,:,1), kkp(:,:,1), etaSymm(:,:,2) ),  title('\eta (high y)')

% calculate how fast total wealth is extracted
KextractRatio = (S(1).cons+S(1).consp) ./ ConsBewleyMat - 1;
figure;
%subplot(1,2,1)
pcolor( kk(MG,MG,1), kkp(MG,MG,1), KextractRatio(MG,MG,1)  )
set(gca,'CLim',[-0.15 .15]), colormap('jet'), colorbar
xlabel('w'), ylabel('w'''), title('dW/dt relative to efficient (low y)')
%subplot(1,2,2)     % don't need second plot here, entirely the same
%pcolor( kk(MG,MG,2), kkp(MG,MG,2), KextractRatio(MG,MG,1)  )
%set(gca,'CLim',[-0.15 .15]), colormap('jet'), colorbar
%xlabel('w'), ylabel('w'''), title('dW/dt relative to efficient (high y)')

end



%% Plot bankruptcy hitting times

if DoHitTimes==1                    % Do this only if asked for
    
indD = false(N,N,N3,N4);            % Define the set D that is hit: 
% indD(1,:,:,:) = 1;                % where first player is broke
indD(:,1,:,:) = 1;                  % or/and where second player is broke

S(1).Q.indD = indD;                 % hand this over to object
S(1).Q.T    = 100;                  % Set highest hitting time to be considered.
                                    % Caution: 100 takes several minutes!
S(1).Q.dt=0.1;                      % Set time increment

S = GetDistHitTimes(S);             % Get distribution of hitting times 
                                    % with extra function
                                    
Q = S(1).Q.cdf;                     % read out cdf of hitting times
q = S(1).Q.pdf;                     % read out pdf
tvec= S(1).Q.t;                     % time vector for which calculations 
N = S(1).Q.N;                       % were made and its length N

% Plot pdf and cdf of hit times for one point in state space
kind  = 40;                     % put index of her asset position on grid here
kpind = 5;                      % same for hime
kk = kgrid(kind);  kkp = kgrid(kpind);

figure;              

subplot(2,1,1)                  % figure for pdf
q1 = squeeze(q(kind,kpind,1,1,:));
q2 = squeeze(q(kind,kpind,2,1,:));
plot(tvec,q1,'-b',tvec,q2,'-g'),    legend('y low','y high')
xlabel('\tau (in years)'),  ylabel('f(\tau)')
title(sprintf('Hitting time of his bankruptcy at w=%1.0fK, w''=%1.0fK',[kk kkp]))

subplot(2,1,2)                  % figure for cdf
Q1 = squeeze(Q(kind,kpind,1,1,:));  Q2 = squeeze(Q(kind,kpind,2,1,:));
plot(tvec,Q1,'-b',tvec,Q2,'-g'),    legend('y low','y high')
xlabel('\tau (in years)'),  ylabel('F(\tau)')


% Quantile plots:
% Plot quantiles of hitting times for the entire state space and both of
% her income levels (10%, 50%, 90% and mean)
figure;
nRow = 4;  nCol = 2;
lev = [1 5 10 20 30];               % set levels for contour lines (in years)
xmax=topK;  ymax = topK;            % set top limits for x- and y-axis

subplot(nRow,nCol,1)
[C,h] = contour(kgrid,kgrid,S(1).Q.p10(:,:,1),lev);  clabel(C,h)
title('p10 for low y'),  ylim([0 ymax]),  xlim([0 xmax])

subplot(nRow,nCol,2)
[C,h] = contour(kgrid,kgrid,S(1).Q.p10(:,:,2),lev);  clabel(C,h)
title('p10 for high y'), ylim([0 ymax]),  xlim([0 xmax])

subplot(nRow,nCol,3)
[C,h] = contour(kgrid,kgrid,S(1).Q.p50(:,:,1),lev);  clabel(C,h)
title('p50 for low y'), ylim([0 ymax]),  xlim([0 xmax])

subplot(nRow,nCol,4)
[C,h] = contour(kgrid,kgrid,S(1).Q.p50(:,:,2),lev);  clabel(C,h)
title('p50 for high y'), ylim([0 ymax]),  xlim([0 xmax])

subplot(nRow,nCol,5)
[C,h] = contour(kgrid,kgrid,S(1).Q.mean(:,:,1),lev);  clabel(C,h)
title('mean for low y'), ylim([0 ymax]),  xlim([0 xmax])

subplot(nRow,nCol,6)
[C,h] = contour(kgrid,kgrid,S(1).Q.mean(:,:,2),lev);  clabel(C,h)
title('mean for high y'), ylim([0 ymax]),  xlim([0 xmax])

subplot(nRow,nCol,7)
[C,h] = contour(kgrid,kgrid,S(1).Q.p90(:,:,1),lev);  clabel(C,h)
title('p90 for low y'), ylim([0 ymax]),  xlim([0 xmax])

subplot(nRow,nCol,8)
[C,h] = contour(kgrid,kgrid,S(1).Q.p90(:,:,2),lev);  clabel(C,h)
title('p90 for high y'), ylim([0 ymax]),  xlim([0 xmax])

% Movie of Q-function:
% figure;
% subplot(2,1,1), surf(Q(:,:,1,1)), zlim([0 1])
% subplot(2,1,2), surf(Q(:,:,2,1)), zlim([0 1])
% set(gca,'nextplot','replacechildren');
% MM(1) = getframe;
% for i=2:N
%     subplot(2,1,1), surf(Q(:,:,1,i)), zlim([0 1])
%     title(sprintf('Distribution of hitting times \\tau: Prob(\\tau<%1.1f)',tvec(i)))
%     subplot(2,1,2), surf(Q(:,:,2,i)), zlim([0 1])
%     MM(i) = getframe;
% end


% Extreme hitting times:
% plot Q-function for latest hitting time with extreme probabilities
lev = [.00001 .0001 .001 .01 .1 .5 .9 .99 .999 .9999];
figure                           
[C,h] = contour(kgrid,kgrid,S(1).Q.cdf(:,:,1,end),lev);  clabel(C,h)
title(sprintf('Prob(he bankrupt within %1.0f years)',S(1).Q.T))

end


display('DONE!')
