function SS = BewleyPolicyFct(S)   
% Gets policy for Bewley model given a value-function guess.
%
% Input:
% Object S with required fields:
%    S(1).V.val:                    value function (e.g. newest guess)
%    S(1).options.UpWindDer:        1 or 0. If 1, use upwind derivatives.
%                                   If 0, use centered derivatives.
%    other fields:                  see lines 17-19
%
% Output:
% Object SS with updated fields:
%    S(1).cons:                     new consumption policy
%    S(1).a:                        drift of assets under new policy
%    S(1).FU:                       flow utility under new policies
%    S(1).V.jac(:,:,1,1):           derivative of value function
%    S(1).V.SecDer(:,:,1,1):        second derivative of value function
%
% Calls:
% GetJac.m
%
% Is called by:
% ValueFctIter.m
%
% Usage:
% S = BewleyPolicyFct(S);           % call directly
%   % or
% PolFctHdl = @BewleyPolicyFct;     % call using a function handle
% S = feval(PolFctHdl,S);


params = S(1).params;               % read out parameters
g = params.g;  r = params.r;  u = params.u;
kk = S(1).xGrid;  yy = S(2).xGrid;
if isfield(S(1),'options')          % see if option 'UpWindDer' is chosen:
    if isfield(S(1).options,'UpWindDer')
        UpWindDer = S(1).options.UpWindDer;
    else                            % if not, do centered differencing
        UpWindDer = 0;
    end
else
    UpWindDer = 0;
end

S = GetJac(S,1,1);                  % calculate derivative of value fct.
                                    % (this extrapolates V automatically!)
Vk = squeeze(S(1).V.jac(:,:,1,1));  % read out derivative of V in k-direction,
                                    % squeeze out 2 dimensions in the end.

if UpWindDer==1                     % If upwind derivatives are asked for:
    cUp = Vk.^(-1/g);               % Get consumption when looking upward.
    S(1).aUp = r*kk + yy - cUp;     % put this as upward law of motion
    cDn = [cUp(1,:); cUp(1:end-1,:)]; % get downward-looking law of motion
    S(1).aDn = r*kk + yy - cDn;     % store as downward law of motion
    S(1).a = GetUpWindLOM(S,1);     % get upwind law of motion (LOM)
   
else
    S = GetJacCnt(S,1,1);           % otherwise use centered derivatives
    Vkc = S(1).V.JacCnt(:,:,1,1);
    c  = Vkc.^(-1/g);               % get consumption
    S(1).a = r*kk + yy - c;         % put into law of motion
end

S(1).a(1,:) = max( 0, S(1).a(1,:) );% set constrained agents to zero consum-
                                    % ption if necessary
c = r*kk + yy - S(1).a;             % back out consumption again
S(1).cons = c;                      % put consumption...
S(1).FU   = u(c);                   % ...and flow utility into S

SS = S;                             % return object
