% This script solves a standard consumption-savings problem (Bewley) using
% value-function iteration (function 'ValFctIter'), plots an exaple history
% for one agent (function 'GetHistory') and solves for the stationary
% density (function 'GetStatDensity').

%% Preparations to use finite-element object
clear S options params;         % clear the object if still there
addpath('..','../SmallFct');    % need all functions for finite-element
                                % object on path: are in parent folder


%% Set up object
% Set options for value-function iteration:                        
epsConv = 10^(-7);      % convergence criterion
maxIter = 5000;         % maximum of iterations
ShowEvery = 1000;       % message after every ... iterations
Howard = 0;             % which type of algorithm (see ValFctIter) 
                        % [Note: Howard=1 (Howard improvement algorithm)
                        %  gives wrong policy on top of grid since no
                        %  exptrapolation is used.]
Extend = 1;             % extrapolate out of grid on top margin
tUpdatePol = 2;         % update every ... time units (see ValFctIter).
dtVar = 1;              % 1: let dt vary in value-function iteration
dt = 0.1;               % if dtVar=0: this is dt for value-fct. iteration
UpWindDer = 1;          % 1: take upwind derivatives, 0: take centered der.

N  = 100;               % number of grid points
linGrid = 1;            % is 1 if linear grid is desired
LogGridPar = .3;        % for "log grid": the closer to 0, the closer to 
                        % true log grid. 

display(' ')            % put empty line, show grid size
display(sprintf('Grid size: N = %1.0f',N))
topK = 1000;            % top level of assets on grid

 
% Set model parameters
g   = 2;                % curvature of utility
u   = @(c) c.^(1-g) / (1-g); % function handle for utility
rho = 0.04;             % rate of time preference
r   = 0.01;             % interest rate
sigma = 0.01;           % risk in savings: 0.01 is 1% standard deviation of
                        % return.
yvec = [12  26   43]';  % income vector to get ages 30-65 earnings distribution in 
                        % US right (mean and variance)
lambda = 0.0995;        % transition parameter to get autocorrelations right
h = [-lambda            lambda    0;
     (1/2)*lambda      -lambda    (1/2)*lambda;
     0                  lambda    -lambda          ];
Ny  = length(yvec);     % length of income vector

% Assign the function that calculates policies to the object
S(1).PolicyFctHdl = @BewleyPolicyFct;   

% hand over parameters to S
params.g = g;  params.r = r;  params.rho =  rho;  
params.u = u;  params.sigma = sigma;
S(1).params = params;       

% hand over algorithm options
options.Howard = Howard;  options.tUpdatePol = tUpdatePol;
options.maxIter = maxIter; options.ShowEvery = ShowEvery;
options.epsConv = epsConv; options.pmmin = 0.3;   options.dtVar = dtVar;
options.dt = dt; options.UpWindDer = UpWindDer;   options.Extend = Extend;
S(1).options = options;     

if linGrid==1                          % Grid:
    kgrid = (linspace(0,topK,N))';     % build linear k-grid
else                                   % or log-style grid
    Abs0 = -LogGridPar*yvec(1)/r;     
    kgrid = (GetLogGrid(0,topK,Abs0,N))'; % make logarithmic grid
end

% Put fundamental info into S.
S(1).nPlr    = 1;          % only 1 agent here: not a game
S(1).GridVec = kgrid;      % Grid vectors for continuous dimension: assets
S(2).GridVec = yvec;       % Discrete dimension 2: income.
S(2).discr   = 1;          % Tell object this is a discrete dimension.
S(2).h       = h;          % Jump-hazard matrix between y's.


S = FinElObjSetUp(S);       % Set up the object

kk = S(1).xGrid;            % read out grids
yy = S(2).xGrid;
S(1).s = sigma*kk;


%% Initial guess for value function V
Vguess = GetBewleyVGuess(S);        % get guess for value function.
S(1).V.val(1:N,:) = Vguess;         % and put it into S.
S = GetJac(S,1,1);                  % Extrapolate value function to
                                    % [points, get grid


%% Value-function iteration
S = ValueFctIter(S);                % do value-function iteration.


%% Get an example history
x0 = [2; 1];                        % initial state for history
dt = 0.1;                           % time increment for simulation.
T  = 20;                            % end of history
N  = T/dt;                          % number of points in time

shocks = NaN*zeros(2,N);            % set up array for income shocks:
t1 = floor(N/3);                    % set time for first shock
t2 = floor(0.75*N);                 % and second shock
shocks(2,1:t1-1)  = 1;              % set low income in beginning, then
shocks(2,t1:t2-1) = 2;              % go up to middle income, and come
shocks(2,t2:end)  = 1;              % back to low again

hist = GetHistory(S,x0,dt,T,shocks); % use function to get history

cvec = interpn(S(1).xGrid,S(2).xGrid,S(1).cons,hist(1,:),yvec(hist(2,:))','spline');
tvec = 0:dt:T;                      %plot history of consumption and income
figure;
subplot(2,1,1)
plot(tvec,yvec(hist(2,:)),tvec,cvec)
legend('y','c')
title('A history')
ylabel('y, c')
subplot(2,1,2)
plot(tvec,hist(1,:))                % plot history for assets k
legend('k')
xlabel('t'), ylabel('k')

%% Plot laws of motion for assets
kk = S(1).xGrid;  yy = S(2).xGrid;  % get grids
c = r*kk + yy - S(1).a;  

figure,                             % plot law of motion
for i=1:Ny                          % loop over all income states
    subplot(2,Ny,i)
    plot(kk,S(1).a(:,i),'b-');      % plot law of motion S.a
    hold on, plot([kk(1) kk(end)],[0 0],'k--')  % mark zero savings
    xlim([0 topK]);
    if i==1
       ylabel('law of motion') 
    end
    title(sprintf('y=%1.2f',yvec(i)));
end



%% Get stationary density
nStat = GetStatDensity(S);      % obtain stationary density and plot
S(1).nStat = nStat;

for i=1:Ny                      % loop over all income levels
    subplot(2,Ny,Ny+i)
    plot(kk,nStat(:,i),'-');    % plot density
    xlim([0 topK]);
    if i==1
        ylabel('density')
    end
    xlabel('k')
end


%% Display some macro statistics
nkMarg = sum(nStat,2);                      % get marginal density over k
display(sprintf('Broke: %1.1f percent',nkMarg(1)*100)) % show % of broke agents
Cpl = kgrid' * nkMarg;
display(sprintf('K = %1.2f',Cpl))           % show capital stock
GDP = 15 + r*Cpl;
display(sprintf('GDP = %1.2f',GDP))         % show GDP
display(sprintf('K/GDP = %1.2f',Cpl/GDP))   % show capital over GDP 

display('DONE!')
