% Simple example: Visualize a diffusion. A particle moves in
% two-dimensional space with drift (a1,a2) and standard deviations (s1,s2)
% in the shocks. The third dimension is discrete: There are two states, 
% between which there is a jump hazard h. Cell 1 sets up the state-space
% object, cell 2 forward-iterates the density and cell 3 draws a sample
% path. The other cells may be skipped on first reading: They concern the
% distribtion of hitting times for certain sets.

%% Set up object 
clear S;                        % clear the object (if still existent)
addpath('..','../SmallFct');    % need all functions for finite-element
                                % object on path: are in parent folder!

a1 = 2;                         % drift of dimension 1
a2 = 1;                         % drift of dimension 2
s1 = 3;                         % volatility in dimension 1
s2 = 3;                         % volatility in dimension 2
hh = 0.2;                       % hazard rate of jumping in dim. 3

N = 50;                         % grid size for dimensions 1 and 2
N3= 2;                          % grid size for dimension 3: Two discrete
                                % between which the system can switch

pmmin = 0.4;                    % The minimal tolerated probability of 
                                % staying at the same state (for
                                % probabilistic interpretation of forward
                                % matrix).
nIter = 100;                    % Number of iterations for mapping forward
                                % the density.

                                % Put fundamental info into S:
grid = (1:N)';                  % grid for dimensions 1 and 2                               
S(1).GridVec = grid;            % Grid vectors for continuous dimensions
S(2).GridVec = grid;            % 1 and 2.
S(3).GridVec = (1:N3);          % Discrete dimension 3.
S(3).discr   = 1;               % tell S that dimension 3 is discrete.
S(1).a = a1;                    % Set drifts and volatilities 
S(1).s = s1;                  
S(2).a = a2;
S(2).s = s2;
S(3).h   = [-hh hh; hh -hh];    % 2x2 matrix of hazard rates for jumps.
                                % h(j,k) gives the hazard rate of
                                % jumping from state j to state k. The
                                % elements on the diagonal have to be 
                                % the sum of the hazards of leaving
                                % state j times -1 (see documentation).
                                  
S = FinElObjSetUp(S);           % Set up S: get the object ready


%% Forward-iterate density

S = GetHamMatrix(S);            % get operators K and Kext
K = S(1).K;                     % read out (forward) K-matrix
                                
dt = GetDt(K,pmmin);            % get sensible time increment
% dt = 0.0025;                  % could also set this manually
nSts = S(1).nSts;               % read out total number of states (N*N*N3)
P = speye(S(1).nSts) + K*dt;    % Get (Markov) transition matrix P on
                                % vectorized state space for time
                                % interval of lenght dt (see documentation)

n0 =  zeros(nSts,1);            % set up starting density
% Alternative 1: all concentrated on one point in the middle:
n0(round(N/2)*N + round(N/2) ) = 1;
% Alternative 2: split between 2 points (un-comment to run):
% n0(round(N/2)*N + round(N/2) ) = 0.7;   
% n0(round(N/4)*N + round(N/4) ) = 0.3;


nc = n0;
F = zeros(N,N,N3,nIter);            % F keeps densities over time
for i=1:nIter                       % Loop over time
    nc = P' * nc;                   % and map the density forward.
    F(:,:,:,i) = reshape(nc,[N N N3]);
end

XX = S(1).xGrid(:,:,1);             % read out 2-dimensional grid over 
YY = S(2).xGrid(:,:,1);             % dimensions 1 and 2 for plotting
figure;                             % Now, make movie of how density evolves
subplot(2,1,1);                     % over time
surf(F(:,:,1,round(nIter/5)));       
axis tight;
subplot(2,1,2);
surf(F(:,:,2,round(nIter/5)));
zlim([0 0.03]);
set(gca,'nextplot','replacechildren');
for i=1:nIter
    subplot(2,1,1)
    surf(XX,YY,F(:,:,1,i));
    title(sprintf('Density at t=%1.2f (dt=%1.3f)',[i*dt, dt]))
    zlim([0 0.03]);
    xlabel('x'),  ylabel('y'), %zlabel('z=1')
    subplot(2,1,2)
    surf(XX,YY,F(:,:,2,i));
    zlim([0 0.03]);
    xlabel('x'),  ylabel('y'), %zlabel('z=2')
    M(i) = getframe;
end
% movie(M,1)                          % Play movie: need not do this again
                                      % since it is visible on
                                      % construction of M.
                                  
%% Plot a sample path
x0 = [ round(N/2); round(N/2); 1];
dt = 0.1;
T  = 5;                              % get a sample path
hist = GetHistory(S,x0,dt,T);

figure;

subplot(2,1,1)                        % and plot.
plot(hist(1,:),hist(2,:))
xlim([0 N]), ylim([0 N])
xlabel('x'), ylabel('y')
title('Sample path')

subplot(2,1,2)
plot(0:dt:T,hist(3,:))
ylim([0 3])
xlabel('t')
ylabel('z')


%% Get hitting-time distribution

% First, define the set that is to be hit:
indD = false(N,N,N3);              
% Alternative 1: margins of the grid in dimensions 1 and 2
indD([1 N],:,:) = 1;                % the margins of the state space in 
indD(:,[1 N],:) = 1;                % the continuous dimensions 1 and 2.
% Alternative 2: small set in the middle (un-comment to run):
%indD = false(N,N,N3);              
%indD(25,25,:) = 1;                 
% Alternative 3: circle around the middle (un-comment to run)
% rad = 24;  cent = [25.5  25.5];       % radius and center of circle
% [indx indy] = ndgrid(1:N,1:N);
% indDvec = (indx(:)-cent(1)).^2 + (indy(:)-cent(2)).^2 > rad^2;
% indDmat = reshape(indDvec,[N N]);
% indD = cat(3,indDmat,indDmat);

S(1).Q.indD = indD;                 % hand over the set to object
S(1).Q.T    = 30;                    % Set highest hitting time to be considered.

S = GetDistHitTimes(S);             % get distribution of hitting times

Q = S(1).Q.cdf;                     % read out Q-function (see documentation)
tvec = S(1).Q.t;
N = S(1).Q.N;                       % get length of  Q

%% Make movie of Q-function over time:
figure;
subplot(2,1,1), surf(Q(:,:,1,1)), zlim([0 1])
subplot(2,1,2), surf(Q(:,:,2,1)), zlim([0 1])
set(gca,'nextplot','replacechildren');
MM(1) = getframe;
for i=10:10:N
    subplot(2,1,1), surf(XX,YY,Q(:,:,1,i)), zlim([0 1])
    title(sprintf('Distribution of hitting times \\tau: Prob(\\tau<%1.1f)',tvec(i)))
    subplot(2,1,2), surf(XX,YY,Q(:,:,2,i)), zlim([0 1])
    MM(i) = getframe;
end

%% Plot distributional characteristics of hitting times for entire state
% space:
figure;
subplot(3,2,1), surf(XX,YY,S(1).Q.mean(:,:,1)), title('mean')
subplot(3,2,2), surf(XX,YY,S(1).Q.p10(:,:,1)) , title('p10')
subplot(3,2,3), surf(XX,YY,S(1).Q.p25(:,:,1)) , title('p25')
subplot(3,2,4), surf(XX,YY,S(1).Q.p50(:,:,1)) , title('p50')
subplot(3,2,5), surf(XX,YY,S(1).Q.p75(:,:,1)) , title('p75')
subplot(3,2,6), surf(XX,YY,S(1).Q.p90(:,:,1)) , title('p90')

%% Plot pdf and cdf of hitting time for one point:
xval = 40;              % set x- and y-value of point
yval = 40;

figure;

subplot(2,1,1)           % plot the pdf
qq   = squeeze( S(1).Q.pdf(xval,yval,1,:) );   
plot(tvec,qq)
strTitle = sprintf('Distribution of hitting times for (x,y)=(%1.0f,%1.0f)',[xval yval]);
title(strTitle),  xlabel('\tau'),  ylabel('pdf: f(\tau)')

subplot(2,1,2)           % plot the cdf
QQ   = squeeze( S(1).Q.cdf(xval,yval,1,:) );  
plot(tvec,QQ)
xlabel('\tau'),  ylabel('cdf: F(\tau)')

