function SS = FinElObjSetUp(S)
% This function sets up a state space for use: Coordinates for grid points
% and their indices in all dimensions, empty arrays for value functions and
% derivatives (plus extension of grid for extrapolation)
%
% Input:
% Structure S with fields:
% S(1).nPlr:        scalar. Number of players.
% S(i).GridVec:     nGridi x 1: grid vectors for dimensions 1 to N. 
% S(i).discr:       0-1 variable. Must be set to one for discrete dimensions
%
% Output:
% Structure S with added fields:
% S(1).nDim:        Number of dimensions
% S(1).nGridVec:    nDim x 1. Grid size of state space.
% S(1).nExtGridVec: nDim x 1. Size of extended grid (extrapolations etc.)
% S(1).nSts:        scalar. Total number of grid points (=prod(nGridVec))
% S(1).nExtSts:     scalar. Total number of grid points in the extended 
%                   grid (=prod(nExtGridVec))
% S(1).ContDim:     nContDim x 1. Vector containing indeces of continuous
%                   dimensions.
% S(1).DiscrDim:    nDiscrDim x 1. Vector containing indeces of discrete
%                   dimensions.
% S(1).FullInd:     cell array with full indeces of the (normal) grid.
% S(1).ExtInd:      cell array with full indeces of the extended grid.
% S(1).K:           nSts x nSts. Empty sparse matrix, used later for
%                   forward (especially) and backward operations on
%                   (normal) grid. Filled by GetHamMatrix.m (using the
%                   laws of motion).
% S(1).Kext:        nSts x nExtSts. Empty sparse matrix, used later on
%                   extended grid, especially backward operations. Filled
%                   by GetHamMatrix.m (using the law of motion).
% S(1).Kreflx:      nExtSts x nSts: Filled sparse matrix, created by this 
%                   function. Turns Kext into K as follows:
%                   K = Kext * Kreflx; Is used by GetHamMatrix.m.
% S(1).Kextend:     nExtSts x nSts: Filled sparse matrix, created by this 
%                   function. Extends normal grid to extended grid (puts
%                   zeros in extra points).
% S(1).Kextrap:     nExtSts x nExtSts: Filled sparse matrix, created by
%                   this function. Extrapolates from normal grid points to
%                   the extra grid points (replaces old values there).
%                   Extend and extrapolate as follows: Kextend * Kextrap;
% S(1).V.val        nGrid1+1 x ... x nGridN x nPlr: empty array for value
%                   functions. Is extended by 1 cell on top for the
%                   continuous states (for extrapolated values)
% S(1).V.jac        nGrid1 x ... x nGridN x nPlr x nDim: empty array for
%                   Jacobian of value functions (upward derivative)
% S(1).V.JacCnt     same as V.jac, but for centered derivatives
% S(1).V.ScdDer     nGrid1 x ... x nGridN x nPlr x nDim: empty array for
%                   2nd (own) derivatives of value functions (NO CROSS
%                   DERIVATIVES HERE: THIS IS NOT THE HESSIAN)
% S(i).nGrid        scalar. Grid size for each dimension i
% S(i).nExtGrid     scalar. size of extended grid (with the extrapolation
%                   points in the continuous dimensions).
% S(i).dx           nGrid1 x ... x nGridN array: Upward difference to
%                   next grid point in dimension i. Extrapolated for the
%                   uppermost cell in each dimension.
% S(i).dxc          "centered difference" to next grid point, i.e. average
%                   between upward and downward difference.
% S(i).ind          nGrid1 x ... x nGridN array: Index for grid in
%                   dimension i.
% S(i).xGrid        nGrid1+1 x ... x nGridN array: i-coordinate for grid.
%                   This is the extended grid for continuous dimensions
%                   (need extrapolation points).
%                   Only for continuous dimensions, is equal to S(i).ind
%                   for discrete dimensions. 
% S(i).a            nGrid1 x ... x nGridN array: empty array for drift in
%                   dimension i (only for continuous dimensions; is not
%                   updated if already specified by user before).
% S(i).s            nGrid1 x ... x nGridN array: empty array for volatility
%                   in dimension i (only for continuous dimensions; is not
%                   updated if already specified by user before).
% S(i).h            nGridi x nGridi matrix: empty matrix for transition
%                   hazard rates in dimension i (only for discrete dimen-
%                   sions; is not updated if already specified by user).
%                   Note that the diagonal has to be filled with NEGATIVE 
%                   values, see the documentation.
% S(i).c            1 x 4 vector. Coefficients for extrapolation on top of
%                   continuous dimensions. Is set up by this function.
% S(1).params       Structure, containing subfields with model parameters.
% S(1).options      structure, containing subfields with options for value-
%                   function iteration. See end of this function for
%                   specific fields and default values.
% % S(1).PolicyFctHdl: function handle. This is what ValueFctIter.m uses to 
%                   obtain policies, laws of motions and flow utilities
%                   given the current value function. Is left empty if not
%                   specified.
% S(1).VguessFctHdl: function handle. This is what ValueFctIter.m uses to
%                   get first guess for value function (in 1st pre-run).
%                   Left empty if not specified.
% S(1).VisualizeVFIFctHdl: function handle. ValueFctIter.m calls this to 
%                   visualize progress. Left empty if not specified.
%
% Usage:
% S(1).nPlr    = 2;             % Put fundamental info into S.
% S(1).GridVec = 0:0.1:20;      % Grid vectors for continuous dimensions
% S(2).GridVec = 0:0.1:20;      % 1 and 2.
% S(3).GridVec = 1:2;           % Discrete dimension 3.
% S(3).discr   = 1;
% S = FinElObjSetUp(S);         % Set up S: Add new fields


%% Set up the fundamental fields
nDim = length(S);               % obtain number of dimensions
S(1).nDim = nDim;               % and write into structure.

if ~isfield(S,'nPlr')           % If there's no field for number of players
    nPlr = 1;                   % put default 1.
    S(1).nPlr = 1;
else
    nPlr = S(1).nPlr;           % if set: read out number of players
end

FullInd  = cell(1,nDim);        % set up cell to have all indeces as comma-
                                % separated list, e.g: 1:50,1:50,1:2 to
                                % facilitate indexing for the user.
ExtInd   = cell(1,nDim);        % do the same for the extended grid.                                
nGridVec = zeros(nDim,1);       % create vector with grid size for all      
nExtGridVec = zeros(nDim,1);    % dimensions. Same for the extended grid.

if ~isfield(S,'discr')          % set up empty field 'discr' if user didn't do
   S(1).discr = []; 
end

ContDim = [];                   % Set up vectors for indeces of continuos 
DiscrDim= [];                   % and discrete dimensions.

for i=1:nDim                    
   nGridVec(i) = length( S(i).GridVec );
   S(i).nGrid = nGridVec(i);
   FullInd{i}  = 1:nGridVec(i);
   if isempty(S(i).discr)       % If dimension was not specified to be
        S(i).discr=0;           % discrete, it's continuous.
   end
   if S(i).discr==0             % get size of extended grid and indices to 
      nExtGridVec(i) = nGridVec(i)+1; % for extended grid
      ExtInd{i} = 1:nGridVec(i)+1;
      ContDim = [ContDim i];    % record in vector of continuous dimensions
   else
      nExtGridVec(i) = nGridVec(i);
      ExtInd{i} = 1:nGridVec(i);
      DiscrDim= [DiscrDim i];
   end
   S(i).nExtGrid = nExtGridVec(i);
end

S(1).nGridVec    = nGridVec;    % record grid size
S(1).nExtGridVec = nExtGridVec; % also for extended grid.
S(1).ContDim     = ContDim;     % record vectors with continuous and
S(1).DiscrDim    = DiscrDim;    % discrete dimensions.
S(1).FullInd     = FullInd;     % record the cell array with indeces
S(1).FullExtInd  = ExtInd;      % record cell array with indeces for exten-
                                % ded grid.

nSts         = prod(nGridVec);  % Calculate total number of states
S(1).nSts    = nSts;
nExtSts      = prod(nExtGridVec);% Also for the extended grid.
S(1).nExtSts = nExtSts;


%% Build grid(s)
if nDim==1                      % if there's one dimension:
    S(1).xGrid  = S(1).GridVec; % Grid is just state-space vector
    S(1).ind    = (1:nGridVec(1))';
    S(1).ExtInd = (1:nExtGridVec(1))';
else                            % If there are more dimensions:
    % build grids here for all dimensions:
    % - S.xGrid returns comma-separated list of grid vectors for all dimensions
    % - ndgrid returns a comma-separated list of the nDim grids 
    % - assign this to S.xGrid, itself a comma-separated list:
    %   read: [S(1).xrid, S(2).xgrid, ..., S(nDim).xGrid] 
    [ S.xGrid ]  =  ndgrid(S.GridVec) ;
    % [ S.xExtGrid ] = ndgrid(S.ExtGridVec);
    % do the same: index numbers here for the grid points in each dimension
    % (starts at 1, goes up to nDim(i) for dimension i)
    [ S.ind   ]  =  ndgrid(S(1).FullInd{:});
    % Do the same for the extended grid:
    [ S.ExtInd ] =  ndgrid(S(1).FullExtInd{:});
end


%% Create fields for grid differences (dx,dxc) and laws of motion (a,s,h):

for i=1:nDim                                % loop over all dimensions i
   nGridi = S(i).nGrid;                     % read out number of states
   
   if S(i).discr==1                         % If state is discrete:
       S(i).dx = ones(nGridVec');           % Set distances of adjacent 
                                            % states to 1.
       S(i).ExtGridVec = S(i).GridVec;      % don't extend grid for discrete
                                            % dimensions.
       S = SetUpField(S,'h',i,zeros(nGridi,nGridi));
                                            % Set up field 'h' for
                                            % transition-hazard matrix
   
   else                                     % for continuous states:
       dxi = diff(S(i).xGrid,1,i);          % 1st difference in i-th dim.
       s   = nGridVec;                      % Want to get upward differen-
       s(i)= 1;                             % ces: Extrapolate differences
       dTop = 2*S(i).GridVec(end)-3*S(i).GridVec(end-1)+S(i).GridVec(end-2);
       dxi = cat(i,dxi,dTop*ones(s'));      % linearly for points with 
       S(i).dx = dxi;                       % highest i-coordinate.
       S(i).ExtGridVec = [S(i).GridVec;  S(i).GridVec(end)+dTop];
                                            % Set extended grid vector
       inddn = FullInd;  inddn{i}=1:nGridi-1; % Get average between up-
       indup = FullInd;  indup{i}=2:nGridi;   % and downward distance for
       dxc   = 0.5*( dxi(inddn{:}) + dxi(indup{:}) ); % centered difference
       ind1  = FullInd;  ind1{i} =1;
       dxcBottom = dxc(ind1{:});            % centered difference for lowest
                                            % grid point: copy the one from
                                            % one grid point above
                                            % (is not used anyway)                                            % this is constraint): put 0.
       dxc = cat(i,dxcBottom,dxc);          % Add values on bottom.
       S(i).dxc = dxc;                      % record in structure.
       
       S = SetUpField(S,'a',i,zeros(nGridVec'));  % Set up fields 'a' and
       S = SetUpField(S,'s',i,zeros(nGridVec'));  % 's' if not filled.
   end
   
end


%% Set up fields for value functions, its derivatives etc.
S(1).V.val    = zeros([nExtGridVec; nPlr]');   % set up arrays for value fct.,
S(1).V.FU     = zeros([nGridVec; nPlr]');      % field for flow utility
S(1).V.jac    = zeros([nGridVec; nPlr; nDim]');% its Jacobian (upward)
S(1).V.JacCnt = zeros([nGridVec; nPlr; nDim]');% centered Jacobian and 2nd
S(1).V.ScdDer = zeros([nGridVec; nPlr; nDim]');% derivatives (NOT the Hessian)



%% Set up empty fields for linear operators

S(1).K       = sparse(nSts,nSts);    % Set up sparse matrix of zeros for K.
S(1).Kext    = sparse(nSts,nExtSts); % set up the extended version


%% Set up matrix Kreflx: This matrix reflects mass from extra grid points
% back into grid. Is used to turn Kext into K in GetHamMatrix.m.
indFrom = (1:nExtSts)';         % indeces from which we're coming on the 
                                % extended grid.
CP = cumprod(nGridVec);
CountUp    = [1; CP(1:nDim-1) ];% How much we have to count up in each 
                                % dimension on vectorized (normal) state
                                % space to reach next state in dimension i.

AllExtInd = {S.ExtInd};         % S.ExtInd: returns comma-separated list of
                                % indeces for all dimensions. {...} packs
                                % this into a structure.
indTo = min( AllExtInd{1}(:), S(1).nGrid(1) );
                                % Create index in vectorized extended state
                                % that would result if one stayed in the
                                % same state, go one down for top grid 
                                % points in all dimensions. First: put in 
                                % index of first dimension.
if nDim>1                       % Then, loop over other dimensions:
    for i=2:nDim                % Have to add CountUp-numbers if 
        NewIndi = min( AllExtInd{i}(:), S(i).nGrid );
        indTo = indTo + (NewIndi -1 ) * CountUp(i); 
    end                         % we're higher than index 1 in respective
end                             % dimension. Again, go down one grid point 
                                % if in top grid point of respective dim.

Kreflx = sparse(indFrom,indTo,ones(nExtSts,1),nExtSts,nSts);
S(1).Kreflx = Kreflx;           % This is the matrix that converts the 
                                % extended K (backward operator) to the
                                % small K (forward operator)




%% Create coefficients for extrapolation on top of grid
Ones = cell(nDim,1);
for i=1:nDim, Ones{i}=1; end

for i=1:nDim                       % loop over all continuos dimensions
if S(i).discr==0    
    N = S(1).nGrid;                     % read out differences between
    ind = Ones;  ind{i} = N-3:N;        % uppermost grid points in 
    dxUp  = squeeze( S(i).dx(ind{:}) ); % dimension i.
    dxN3 = dxUp(1);  dxN2 = dxUp(2); dxN1 = dxUp(3); dxN = dxUp(4);
    dxcUp = squeeze( S(i).dxc(ind{:} ) ); % also centered differences
    dxcN2 = dxcUp(2); dxcN1 = dxcUp(3); dxcN = dxcUp(4);

    aFrst1 = dxN/dxN1;              % coefficient relating to 1st derivative
    aScd1  = dxN*dxcN/dxN1/dxcN1;   % two coefficients relating to 2nd der.
    aScd2  = dxN*dxcN/dxN2/dxcN1;
    bThrd  = dxN*dxcN*dxN1/dxN2;    % helper variable for...
    aThrd1 = bThrd/dxcN1/dxN1;      % ...coefficients relating to 3rd der.
    aThrd2 = bThrd/dxcN1/dxN2;
    aThrd3 = bThrd/dxcN2/dxN2;
    aThrd4 = bThrd/dxcN2/dxN3;
     
    c0 = 1 + aFrst1 + aScd1 + aThrd1;
    c1 = aFrst1 + aScd1 + aScd2 + aThrd1 + aThrd2 + aThrd3;
    c2 = aScd2 + aThrd2 + aThrd3 + aThrd4; 
    c3 = aThrd4;                    % calculate c-coefficents from notes...
    S(i).c = [c0 -c1 c2 -c3];       % and write them into S-field 'c'.
end    
end


%% Create matrix Kextrap (extrapolates by pre-multiplying)

X = speye(nExtSts);                 % create sparse identity matrix of size
                                    % nExtSts: Just copy values of function
                                    % for all states from normal grid.

CP = cumprod(nExtGridVec);
CountUpExt = [1; CP(1:nDim-1)];     % create CountUp for extended grid.

for i=1:nDim                        % Loop over all continuous dimensions
if S(i).discr==0
   if nDim==1                       % Construct array is of size of the extended
       AllExtVecInd = (1:nExtSts)'; % grid: Contains index of each grid
   else                             % point in the vectorized state space
       AllExtVecInd = reshape(1:nExtSts,nExtGridVec');
   end
   iIndExtra = S(1).FullExtInd; iIndExtra{i} = nExtGridVec(i);
                                    % iIndExtra: Indeces of extra grid
                                    % points in dimension i
   ExtraVecInd = AllExtVecInd(iIndExtra{:});
                                    % Get indeces (for vectorized version)
                                    % of these extra points
   X0 = sparse(ExtraVecInd(:),ExtraVecInd(:),-1,nExtSts,nExtSts);
   X = X+X0;

   for j=1:4                        % Loop over the four lower grid points
                                    % we need for extrapolation.
        VecIndj = ExtraVecInd(:) - j*CountUpExt(i);
        Xj = sparse(ExtraVecInd(:), VecIndj(:), S(i).c(j),nExtSts,nExtSts);
        X  = X + Xj;
   end                              % Put the corresponding extrapolation 
end                                 % coefficient on right point.
end

S(1).Kextrap = X;


%% Create matrix Kextend:
% extends vector from normal grid size to extended grid size (putting zeros
% into the newly created points)

indTo = (1:nSts)';

AllInd = {S.ind};               % S.ind: returns comma-separated list of
                                % indeces for all dimensions. {...} packs
                                % this into a structure.

indX = AllInd{1}(:);            % Create index in vectorized extended state
                                % that would result if one stayed in the
                                % same state. First: put in index of first
                                % dimension.
if nDim>1                       % Then, loop over other dimensions:
    for i=2:nDim                % Have to add CountUp-numbers if we're 
        indX= indX + ( AllInd{i}(:) -1 ) * CountUpExt(i); 
    end                         % higher than index 1 in dimension i.
end   
indFrom = indX;

Kextend = sparse(indFrom,indTo,1,nExtSts,nSts);
S(1).Kextend = Kextend;


%% Set up fields for parameters, options and function handles

if ~isfield(S,'params')                 % Create empty field for parameters
    S(1).params.rho = .1;               % if not there. Need at least time
elseif ~isfield(S(1).params,'rho')      % preference rate here. 
    S(1).params.rho = .1;
end

if ~isfield(S,'options')                % Create field 'options' with sub-
    S(1).options.epsConv     = 10^(-10);% fields that contain default
    S(1).options.maxIter     = 10^5;    % options for value-function
    S(1).options.dtVar       = 1;       % iteration (described below)
    S(1).options.dt          = 0.1;
    S(1).options.pmmin       = 0;          
    S(1).options.Howard      = 0;
    S(1).options.tUpdatePol  = 1; 
    S(1).options.Extend      = 0;
    S(1).options.ShowEvery   = 1000;
    S(1).options.VisualizeVFI= 0; 
    S(1).options.Mesage      = 1;       % Even if 'options' exists, might 
else                                    % have to create some sub-fields
    if ~isfield(S(1).options,'epsConv'),    % convergence criterion
    S(1).options.epsConv = 10^(-10); end
    if ~isfield(S(1).options,'maxIter'),    % maximal number of iterations
    S(1).options.maxIter = 10^5;     end 
    if ~isfield(S(1).options,'dtVar'),      % should time increment dt be
    S(1).options.dtVar   = 1;        end    % variabel (1) or not (0)
    if ~isfield(S(1).options,'dt'),         % time increment for iteration
    S(1).options.dt      = 0.1;      end
    if ~isfield(S(1).options,'pmmin'),      % for variable time increment,
    S(1).options.pmmin   = 0;        end    % this is the probability of staying
                                            % at current node that will be
                                            % set.
    if ~isfield(S(1).options,'Howard'),     % Option for type of Howard
    S(1).options.Howard    = 0;      end    % algorithm
    if ~isfield(S(1).options,'tUpdatePol'), % In Howard=2 algorithm, update
    S(1).options.tUpdatePol= 1;      end    % policies every ... time units.
    if ~isfield(S(1).options,'Extend'),     % Jump out of grid and use 
    S(1).options.Extend = 0;         end    % extrapolated values (1) or not.
    if ~isfield(S(1).options,'ShowEvery'),  % show progress in iteration 
    S(1).options.ShowEvery = 1000;   end    % every ... iterations.
    if ~isfield(S(1).options,'VisualizeVFI'), % visualize progress in iter-
    S(1).options.VisualizeVFI=0;     end    % ation (1) or not (0).
    if ~isfield(S(1).options,'Message'),    % show message if converged (1)
    S(1).options.Message=1;          end    % or not (0).
end

if ~isfield(S,'PolicyFctHdl')               % Put in empty fields if the
    S(1).PolicyFctHdl = [];                 % three here are not there yet
end
if ~isfield(S,'VguessFctHdl')
    S(1).VguessFctHdl = [];
end
if ~isfield(S,'VisualizeVFIFctHdl')
    S(1).VisualizeVFIFctHdl = [];
end


%% return S
SS = S;                                     % return set-up state space

