function SS = GetDistHitTimes(S)
% For the set D defined by th logical indeces in S(1).Q.indD, return the
% function Q(t,x) (in S(1).Q.cdf). Q(t,x) is the probability that
% the process given in S (drift S.a, volatility S.s and jump hazards 
% S.h) hits the set D from point x within the time interval[0,t]. See the
% documentation
%
% Input:
% object S with fields that are read out in beginning of code, specifically
% with a structure S.Q. Important fields:
%       Q.indD:     dim1 x dim2 x ... x dimN logical array. Grid points
%                   marked with 1 are in set D, those with 0 are outside.
%       Q.T:        scalar (optional). Length of time for which Q-function 
%                   is to be calculated.
%       Q.dt:       scalar (optional). Time interval between time-values
%                    for which Q is to be calculated.
%
% Output:
% object SS in which the sub-field Q is modified, specifically in fields:
%       Q.N:        scalar. Length of array for Q-function in time
%                   dimension.
%       Q.t:        1 x N row vector. Time values for Q-function.
%       Q.cdf:      dim1 x dim2 x ... x dimN x N array, contains values for
%                   Q-function (time is on last dimension), i.e. the cumu-
%                   lative distribution function of hitting times.
%       Q.pdf:      The pdf of hitting times, obtained by differencing from
%                   Q.cdf.


Q        = S(1).Q;                  % read out fields from object
nSts     = S(1).nSts;
nGridVec = S(1).nGridVec;
K        = S(1).K;

indD = Q.indD;                      % get indices of grid points that  
                                    % define the set D (that is hit by
                                    % process)

if isfield(Q,'dt'),  dt = Q.dt;  else  dt = GetDt(K); Q.dt = dt; end
if isfield(Q,'T'),   T  = Q.T;   else  T  = 200*dt;   Q.T  = T;  end
                                    % If length of time and time interval
                                    % are not set by user, set standard
                                    % values.
                                    
I = 1:nSts;                         % get indices of set D for state space
I = reshape(I,nGridVec');           % in vectorized form (indices are  
indDvec = I(indD);                  % numbers, not logical any more)

N   = 1 + floor(T/dt);              % length of array for Q-function
Q.N = N;                            % record in Q-structure
QQ = zeros(nSts,N);                 % set up array for Q
QQ(indDvec,1) = 1;                  % For t=0, Q=1 for points in D and 0
                                    % else.

H = speye(nSts) + K * dt;             % Create forward matrix: Has probabi-
                                    % lities of jumping to adjacent states
                                    % in rows

for i=2:N                           % loop over t
   QQ(:,i) = H * QQ(:,i-1);         % calculate Q outside D by taking 
   QQ(indDvec,i) = 1;               % expectations, and set it to 1 on D
end                                 % always
% Almost no time difference here when using small helper variables qqold
% and qqnew.

t  = linspace(0,T,N);               % vector with hitting times

qq = [ QQ(:,1),  diff(QQ,1,2) ] ;   % get pdf from cdf of hitting times

Q.t    = linspace(0,T,N);           % record time-vector, values for Q-
Q.cdf  = reshape(QQ,[nGridVec' N]); % function (cdf and pdf)
Q.pdf  = reshape(qq,[nGridVec' N]); 

Qmin = min( QQ(:,N) );              % See how much probality mass is left
if Qmin < 0.99                      % for times larger than T.
   display(sprintf('Highest probability mass remaining at T=%1.2f was %1.3f',[T 1-Qmin])) 
end                                 % Tell user if more than 1%.

QQ(:,N) = 1;                        % for further calculations, set highest
                                    % hitting times to T.

qq = [ QQ(:,1),  diff(QQ,1,2) ] ;   % get modified pdf from cdf of hitting times
                                    
tt = t( ones(nSts,1) , : );         % Expand t-vector to size of state space.
m  = sum( qq.*tt , 2);              % Calculate mean and variance of hitting
mm = m( :, ones(1,N) );             % times.
v  = sum( qq.*(tt-mm).^2 , 2);

qtl = [0.1  0.25  0.5  0.75  0.9];  % Calculate 5 percentiles of the 
pp  = zeros(nSts,length(qtl));      % hitting-time distribution.
for j=1:nSts
    qqj = qq(j,:);
    if qqj(1)<1                     % All percentiles are 0 if all mass on 0.
        pp(j,:) = GetQuantProbDist(t',qqj',qtl);
    end
end

Q.mean = reshape(m, nGridVec');     % record distributional characteristics
Q.var  = reshape(v, nGridVec');     % un object
Q.p10  = reshape(pp(:,1), nGridVec') ;
Q.p25  = reshape(pp(:,2), nGridVec') ;
Q.p50  = reshape(pp(:,3), nGridVec') ;
Q.p75  = reshape(pp(:,4), nGridVec') ;
Q.p90  = reshape(pp(:,5), nGridVec') ;

S(1).Q = Q;                         % put modified structure Q into object

SS     = S;                         % hand back object as result
