function SS = GetHamMatrix(S)
% Obtains the Hamiltonian matrix for the object S.
%
% Input:
% S: structure of length nDim, where nDim is the number of dimensions of 
% the state space, should have been set up by FinElObjSetUp.m (see example
% below). Required Fields:
%       discr:  0-1 variable. Is 1 if dimension i contains a discrete
%               state, 0 if it's a continuous state. 
%       nGrid:  scalar. Number of states for dimension i
%       ind:    nGrid1 x ... x nGridN array, containing the dimension-i 
%               indeces for the entire state space. Use 'ndgrid' function
%               to obtain these... meshgrid switches 1st and 2nd dimension!
%       dx:     nGrid1 x ... x nGridi-1 x ... x nGridN array, containing
%               the (upward) difference between grid points in dimension i.
%               Index 1 in dimension i refers to the difference between
%               grid point 1 and 2 and so forth.
%       a:      nGrid1 x ... x nGridN array, drift in dimension i.
%       s:      nGrid1 x ... x nGridN array, volatility in dimension i.
%       h:      nGridi x nGridi matrix of transition hazard rates for the
%               discrete state pertaining to dimension i. h(j,k) gives
%               the hazard rate of jumping from state j to state k.
%
% Output:
% Updates the following two "Hamiltonian matrices" in S (which operate on
% the vectorized state space):
% S(1).Kext:    nSts x nExtSts sparse matrix (for backward operations:
%               includes the extrapolated grid point on top of continuous
%               dimensions).
% S(1).K:       nSt-by-nSt sparse matrix (for forward operations: Movements
%               that leave state space on top are reflected in).
% % S(1).A:     function handle. Applies A-operator to function. Leave
% %             because it takes a lot of time
%
% Usage:
% The following is how to fill the structure S for an example of a
% 3-dimensional state space:
% The first two dimensions are continuous and have 50 grid points, the
% 3rd dimension is for a discrete jump variable with 2 grid points.
%
% N = 50;
% a1 = 2;  s1=1; hh = 0.1;
% grid = 0:1:N-1;                 % linear grid
% S(1).nPlr    = nPlr;            % Put fundamental info into S.
% S(1).GridVec = grid;            % Grid vectors for continuous dimensions
% S(2).GridVec = grid;            % 1 and 2.
% S(3).GridVec = (1:N3)';         % Discrete dimension 3.
% S(3).discr   = 1;
% S(1).a = a1;                    % Set drifts and volatilities: 
% S(1).s = s1;                    % FinElObjSetUp will understand how to 
% S(2).a = a1;                    % expand the scalars.
% S(2).s = s1;
% S(3).h   = [-hh hh; hh -hh];    % 2x2 matrix of hazard rates for jumps.
%                                 % h(j,k) gives the hazard rate of
%                                 % jumping from state j to state k. The
%                                 % elements on the diagonal have to be 
%                                 % the sum of the hazards of leaving
%                                 % state j times -1 (see documentation).
% S = FinElObjSetUp(S);           % Set up S: Adds new fields etc.
% S = GetHamMatrix(S);            % Get the Hamiltonian matrices
% K = S(1).K;                     % read out K (is the same as the one
%                                 % created by GetHamMatrix)
% Vnew = FU*dt + (1-rho*dt)*Vold + K*Vold*dt
%                                   % update value function (value-function
%                                   % iteration)
% A = rho*speye(nSt) - K;           % Get value function in a
% Vhow = A \ FU;                    % Howard-improvement step.
% P = speye(nSt) + K*dt;            % Get (Markov) transition matrix P on
%                                   % vectorized state space for time
%                                   % interval of lenght dt.
% n10 = (P')^10 * n0;               % map forward density n0 10 periods of
%                                   % lenght dt.


nDim     = S(1).nDim;           % read out number of dimensions,         
nGrid    = S(1).nGridVec;       % normal grid size,
nExtGrid = S(1).nExtGridVec;    % and extended grid size
nSts     = S(1).nSts;           % number of states (not extended)
nExtSts  = S(1).nExtSts;        % number of states (extended grid)

indFrom = (1:nSts)';            % index of all states in vectorized state
                                % space: where jumps come from

CP = cumprod(nGrid);            % CountUp(i) gives the number we have to
CountUp    = [1; CP(1:nDim-1) ];% increase the (vectorized) index to get to
                                % the next-highest cell in dimension i
                                % (keeping all other dimensions constant)
                                % in the normal grid.

CPext = cumprod(nExtGrid);      % create CountUpExt: same for the extended 
CountUpExt = [ 1; CPext(1:nDim-1) ];  % grid.

AllInd = {S.ind};               % S.ind: returns comma-separated list of
                                % indeces for all dimensions. {...} packs
                                % this into a structure.

indStay = AllInd{1}(:);         % Create index in vectorized extended state
                                % that would result if one stayed in the
                                % same state. First: put in index of first
                                % dimension.
if nDim>1                       % Then, loop over other dimensions:
    for i=2:nDim                % Have to add CountUp-numbers if we're 
        indStay = indStay + ( AllInd{i}(:) -1 ) * CountUpExt(i); 
    end                         % higher than index 1 in dimension i.
end                            
                                
K = sparse(nSts,nExtSts);       % Set up empty nSts-by-nExtSts sparse 
                                % matrix to for result

                                
for i=1:nDim                    % loop over all dimensions i
    niGrid = S(i).nGrid;        % read out grid size of dimension i
    indi  = S(i).ind(:);        % read out index of dimension i as vector 
    
    if S(i).discr==0            % If dimension is a continuous variable:
        ai = S(i).a(:);         % read out drift and volatility for dimen-
        s2i= (S(i).s(:)).^2;    % sion i
        
        dxiup = S(i).dx(indFrom);
                                % Get the upward difference to the next
                                % grid point in the i-direction.
        indDxDn = indFrom - (indi>1).*CountUp(i);
        dxidn = S(i).dx(indDxDn); % Analogous for downward jumps: If we're
                                % not in the lowest state of dimension i,
                                % go down in grid in i-direction and find 
                                % index of next-lowest cell. Read out
                                % downward difference to next grid point in
                                % i-direction.
        dxim  = 0.5*(dxiup+dxidn);
                                % Get average between upward and downward
                                % difference (for 2nd derivative).
        caiup = max(ai,0) ./ dxiup;
        caidn = max(-ai,0)./ dxidn;
        csiup = 0.5*s2i ./ (dxiup.*dxim);
        csidn = 0.5*s2i ./ (dxidn.*dxim);
                                % Calculate coefficients for drift and
                                % volatility terms in dimension i.                           
        indUp = indStay + CountUpExt(i);
                                % Go up in grid in i-direction, i.e. add
                                % Count-up number. Note that one can always
                                % go one up in the extended state space!
        indDn = indStay - (indi>1).*CountUpExt(i);
                                % Go one down in i-direction (except when
                                % we're already on bottom: stay then).

        KiUp  = sparse(indFrom,indUp,caiup+csiup,nSts,nExtSts);
                                % Create sparse matrix summing up the
                                % upward coefficients.
        KiDn  = sparse(indFrom,indDn,caidn+csidn,nSts,nExtSts);
                                % Create matrix summing downward coeff.
        KiM   = sparse(indFrom,indStay,-caiup-caidn-csiup-csidn,nSts,nExtSts);
                                % Create matrix summing middle coeff.
        K = K + KiUp + KiDn + KiM;   
                                % Add the three matrices to what we already
                                % have
        
    else                        % If dimension i is a discrete variable
        for j=1:niGrid          % loop over all states from which one can
                                % jump: j
        indFromj = indFrom(indi==j);
        indStayj = indStay(indi==j);
                                % Get indeces of all cells that have state
                                % i equal to j                        
           for k=1:niGrid       % loop over all states to which one can 
                                % jump: k
                indTo    = indStayj + (k-j)*CountUpExt(i);
                                % Get index of cell we're jumping to
                Kijk = sparse(indFromj,indTo,S(i).h(j,k),nSts,nExtSts);
                                % Write the hazard rate from matrix
                                % h (hazard rates for jumps) into all
                                % cells in matrix. This is a negative
                                % number for the "jump" j-to-j.
                K = K + Kijk;   % Add this matrix to what we already have
           end
        end
    end
    
end

S(1).Kext = K;                  % record the extended matrix
S(1).K    = S(1).Kext * S(1).Kreflx;
                                % convert to the smaller version with
                                % conversion matrix that FinElObjSetUp.m
                                % has calculated.
                                
%S(1).A = @(f) reshape( S(1).K*f(:) , nGrid' );
                                % create function handle that applies
                                % A-operator (infinitesimal generator) to a
                                % function (i.e. array) f. This takes very
                                % long, so leave out to save time.
                                
SS = S;                         % return S-object
