function out = GetHistory(S,x0,dt,T,shocks)
% Simulates one history of the system from t=0 to t=T, starting at state
% x0 and taking time intervals dt for the approximations. This is for
% instructive purposes: Use density iteration instead of simulation to
% obtain stochastic properties of the system. Also, note that the process
% is kept within the grid given in S reflecting back at the boundaries.
% 
% Inputs:
% S:        object with state space etc.
% x0:       nDim x 1 vector. Initial state of system. Give the index of the
%           state for discrete dimensions (1,...,nGridi), not the value
%           itself (e.g. 38 from state vector [17, 25, 38]);
% dt:       scalar. Time increment for algorithm
% T:        scalar. Stop time for algorithm
% shocks:   optional, nDim x T/dt matrix. Shocks to system. A sequence of
%           natural numbers (1,...,nGridi) for discrete states, a sequence
%           of N(0,1)-distributed random variables for continuos
%           dimensions.
%
% Output:
% nDim x T/dt+1 matrix, history for dimension i is in row i.

nDim = S(1).nDim;
ContDim = S(1).ContDim;                 % read out vector of continous 
DiscrDim= S(1).DiscrDim;                % and discrete dimensions                                     
FullInd = S(1).FullInd;                 % read out full set of indices

N    = floor(T/dt);                     % obtain number of shocks necessary
if nargin<5
    shocks = NaN(nDim,N);               % set up shock matrix if not given
end

hist = NaN(nDim,N+1);                   % set up matrix for history


for i=1:nDim                            % draw shocks for all dimensions
if isnan(shocks(i,1));                  % if no shock specified for dim. i:
    if S(i).discr==1                    % For discrete dimensions:
        h    = S(i).h;                  % read out transition-hazard matrix
        xvec = 1:S(i).nGrid;            % get grid indices
        xold = x0(i);                   % read out starting value in dim. i
        hist(i,1)   = xold;             % set as first value of history
        for j=1:N                       % loop over time:
            n = h(xold,:)*dt;           % read out jump probabilities to 
            n(xold) = 1+n(xold);        % other states, adjust value for 
                                        % staying in same state
            xnew = GetDrawProbDist(xvec,n(:)); % draw new state
            shocks(i,j) = xnew;         % record new state: shock in discrete
                                        % dimension is just the index of
                                        % next t's state.
            xold = xnew;                % re-set for next iteration
        end                             
        hist(i,2:end) = shocks(i,:);    % history equals shock history for 
                                        % discrete dimensions 
    else                                % if continuous dimension:
        shocks(i,:) = randn(1,N);       % draw normal (i.e. Brownian motion 
    end                                 % shocks) -- adjust the variance 
else                                    % later
    if S(i).discr==1
       hist(i,1) = x0(i);               % if shock is given, write them 
       hist(i,2:end) = shocks(i,:);     % directly into history for discrete
    end                                 % dimensions.
end                                  
end

nContDim = length(ContDim);
ContGrids = cell(1,nContDim);           % Read grids of continuous variables.
xold = cell(1,nContDim);                % Create cells for current conti-
xnew = xold;                            % nuous variables -- don't
                                        % vectors here because we want to
                                        % create comma-separated lists to
                                        % use 'interpn.m'

indc = FullInd;                         % Cell for indexing:
for k=DiscrDim                          % loop over discrete states:
    indc{k} = 1;                        % set index in discrete dimension
end                                     % to 1 and...
                                       
for i=ContDim                           % ...create grid only over continuous
    iGrid = squeeze( S(i).xGrid(indc{:}) ); % dimensions. Squeeze out
    ContGrids{i} = iGrid;               % discrete dimensions.
    xold{i} = x0(i);      
    hist(i,1) = x0(i);
end


for j=1:N                               % main loop over time: Given shocks, find 
    indc = FullInd;                     % the history for continous dimensions.
    for k=DiscrDim                      % loop over discrete states:
       indc{k} = hist(k,j);             % set index in discrete dimension
    end                                 % to current state.
    
    for i=ContDim                       % Loop over continuos dimensions             
                                        % dimensions
        % Read out grid with drift for dimension i (only for current state
        % of continuous dimensions, squeeze out discrete dimensions)
        aigrid = squeeze( S(i).a(indc{:}) ); 
        % The same for the volatility in dimension i:
        sigrid = squeeze( S(i).s(indc{:}) );
        if nContDim==1                 
        % get current drift by interpolation, same for volatility: 
            aic = interp1(ContGrids{:},aigrid,xold{:},'linear'); % ({:}-indexing
            sic = interp1(ContGrids{:},sigrid,xold{:},'linear'); % of cells
        else                                                     % gives comma-
            aic = interpn(ContGrids{:},aigrid,xold{:},'linear'); % separated 
            sic = interpn(ContGrids{:},sigrid,xold{:},'linear'); % lists)
        end
                
        % Find next value in dimension i given shock:        
        xnew{i} = xold{i} + aic*dt + sic*sqrt(dt)*shocks(i,j);
        % Keep process within the grid for continuous dimension:
        xnew{i} = max( xnew{i}, S(i).GridVec(1)   );
        xnew{i} = min( xnew{i}, S(i).GridVec(end) );
        % Record new value in history and continue loop:
        hist(i,j+1) = xnew{i};
    end
    xold = xnew;    
end

out = hist;                             % hand over history
