function SS = GetJac(S,PlrVec,DimVec,ExtrapV)
% This function calculates numerical derivatives of the value function of
% the players with indeces in PlrVec in the dimensions in vector DimVec.
% Calculates entire Jacobian (for non-discrete dimensions) if 2nd and 3rd
% argument are omitted.
%
% Also computes second derivatives in these dimensions and fills
% S(1).V.ScdDer with it. Linearly extrapolates 2nd derivative on grid
% boundaries and uses this to extrapolate Jacobian. The value function is
% also extrapolated if desired (i.e. ExtrapV=1).
% The extrapolated value in dimension i is written int S(1).V.val in the
% top grid point. See documentation for details on extrapolation procedure.
% 
% Input:
% S: structure with following fields (are all filled if 'FinElObjSetUp.m' 
% has been used on S before, only have to fill S(1).V.val with value
% functions):
%   S(1).nDim:        scalar. Number of dimensions.
%   S(1).nPlr:        scalar. Number of players.
%   S(1).nGridVec:    vector: nDim x 1. Grid size for all dimensions.
%   S(i).dx:          array: nGrid1 x ... x nGridnDim. Difference of grid
%                     points in dimension i.
%   S(i).dxc:         array: nGrid1 x ... x nGridnDim. Centered difference
%                     of grid points in dimension i.
%   S(1).V.val:       array: nGrid1+1 x ... x nGridnDim x nPlr. Value functions
%                     for all players, extended by one grid point on top to 
%                     accomodate extrapolated values.
%   S(1).V.jac:       array: nGrid1 x ... x nGridnDim x nPlr x nDim.
%                     Jacobian of value functions.
%   S(1).V.ScdDer:    array: nGrid1 x ... x nGridnDim x nPlr x nDim. Second
%                     derivative of value functions. NOT THE HESSIAN, no
%                     cross derivatives here!
% PlrVec: 1 x n vector, default is 1:S(1).nPlr. Gives the numbers of the
%         players for which Jacobian is to be calculated. Calculate for
%         all players if omitted.
% DimVec: 1 x k vector, default is 1:S(1).nDim. Gives the indeces of the
%         dimensions in which Jacobian is to be calculated. Calculate for
%         all dimensions if omitted.
% ExtrapV:scalar, default is 1. If 1, the function extrapolates the value
%         function outside regular grid and replaces the old values
% 
% Output;
% Structure S, like input but with updated V.jac, V.ScdDer and V.val.


if nargin < 4                       % If user does not say if to extrapolate
   ExtrapV = 1;                     % the value function, then do it
end

if nargin < 3                       % If indeces for dimensions not given:
    DimVec = 1:S(1).nDim;           % take all dimensions.
end
if nargin < 2                       % If indeces for players not given:
    PlrVec = 1:S(1).nPlr;           % take all players.
end

nGridVec = S(1).nGridVec;           % size of grid

FullInd = S(1).FullInd;             % get cell array of indeces of all
                                    % states: {1:N1,1:N2,1:N3,...}

for ii=1:length(DimVec)             % loop over all dimensions we need:
i = DimVec(ii);                     % i: index of dimension.
                                    % space: i
if S(i).discr == 0                  % Only do this for continuos dimensions
    N = nGridVec(i);
    indDxi    = FullInd;  indDxi{i}     = 1:(N-1);
    indCentDxi= FullInd;  indCentDxi{i} = 2:(N-1);
    indFrstXi = FullInd;  indFrstXi{i}  = 1;
    indSecdXi = FullInd;  indSecdXi{i}  = 2;
    indThrdXi = FullInd;  indThrdXi{i}  = 3;
    indNXi    = FullInd;  indNXi{i}     = N;
    indN1Xi   = FullInd;  indN1Xi{i}    = N-1;
    indN2Xi   = FullInd;  indN2Xi{i}    = N-2;
    indNNXi   = FullInd;  indNNXi{i}    = N+1;
                                    % get some indeces we will need
    
    dxi = S(i).dx(indDxi{:});       % read out upward distances between 
                                    % grid points in i-direction
    TopDxi = S(i).dx(indNXi{:});    % get difference of last point in i-grid 
                                    % to the extrapolation point (outside
                                    % grid).
    CentDxi    = S(i).dxc(indCentDxi{:});
    TopCentDxi = S(i).dxc(indNXi{:}); % the same for the centered distances
     
    for jj=1:length(PlrVec)         % Loop over all players we need:
       j = PlrVec(jj);              % j: index of current player.

       vv = S(1).V.val(FullInd{:},j); % read out value function of j
                                    % the extrapolated grid points on top
                                    % of non-discrete dimensions).
       FrstDer = diff( vv,1,i) ./ dxi;   % Take 1st (upward) numerical
                                    % derivative in dimension i
       SecdDer = diff(FrstDer,1,i) ./ CentDxi;
                                    % Calculate second (centered) derivative 
                                    % for all interior grid points
       SecdDer = cat(i,SecdDer(indFrstXi{:}),SecdDer,SecdDer(indN2Xi{:}));
                                    % For first and last grid point, first
                                    % copy 2nd der. from neighboring point.
       TopThrdDer = ( SecdDer(indN1Xi{:}) - SecdDer(indN2Xi{:}) ) ./ dxi(indN2Xi{:});
       CorrUp     = TopThrdDer .* dxi(indN1Xi{:});
       SecdDer(indNXi{:}) = SecdDer(indNXi{:}) + CorrUp;
                                    % Then correct: linearly extrapolate
                                    % 2nd derivative to top grid point
       BotThrdDer = ( SecdDer(indThrdXi{:}) - SecdDer(indSecdXi{:}) ) ./ dxi(indSecdXi{:}) ;
       CorrDn     = BotThrdDer .* dxi(indFrstXi{:}) ;
       SecdDer(indFrstXi{:}) = SecdDer(indFrstXi{:}) - CorrDn;
                                    % the same on the bottom.
       S(1).V.ScdDer(FullInd{:},j,i) = SecdDer;
                                    % record 2nd der. in object.
       TopFrstDer = FrstDer(indN1Xi{:}) + SecdDer(indNXi{:}) .* TopCentDxi;                             
                                    % Get Jacobian for top row by linearly
                                    % extrapolating, see notes.
       FrstDer = cat(i,FrstDer,TopFrstDer);  % add this to FrstDer,
       S(1).V.jac(FullInd{:},j,i) = FrstDer;   % and record in object
       
       if ExtrapV==1                % If extrapolation of value fct. is desired:
           TopV = S(1).V.val(indNXi{:},j) + FrstDer(indNXi{:}) .* TopDxi;
                                    % Extrapolate value function outside
                                    % the grid using the extrapolated Jac'n
           S(1).V.val(indNNXi{:},j) = TopV;
       end                          % Write this into the extended value
                                    % function.
    end       
              
end
end

SS = S;                             % return state-space structure
