function out = GetMargDensity(S,dimVec)
% Obtains the marginal densities for the stationary density nStat.
%
% Input:
% object S with fields:
%   S(1).nStat: nGrid1 x nGrid2 x ... x nGridN array with density (sums 
%               to 1).
%   S(1).nDim:  scalar. Number of dimensions.
%   S(1).nGridVec: nDim vector. Size of grid (normal grid, not extended
%               one)
% dimVec:       optional. 1-by-k row vector, giving the dimensions for
%               which marginal density should be computed (note: Setting 
%               dimVec = [1 2] integrates out dimensions 3,...,nDim
%               for EACH point (x_1,x_2), resulting in the marginal density
%               f(x_1,x_2)
% Output:
% - if only S is handed over: object S with updated field (for all
%   dimensions i): S(i).nMarg: nGridi x 1 vector with marginal density of
%   dimensions)
% - if also dimVec is handed over: the marginal density f, an nDim1 x ..
%   nDimk array (k is the length of dimVec).
%
% Usage:
% S = GetStatDensity(S);            % Get stationary density for object S
% S = GetMargDensity(S);            % Get the marginal densities.
% plot(S(2).xGrid,S(2).nMarg)       % Plot marginal density for dimension 2
%   or:
% xyDens = GetMargDensity(S,[1 2])  % get marginal density for 2 variables
% surf( xGrid, yGrid, xyDens ); % make surface plot of x/y distribution 

nDim = S(1).nDim;                   % Read out number of dimensions,
nGridVec = S(1).nGridVec;           % and dimension of grid.

if nargin<2
    for i=1:nDim                    % Loop over all dimensions
        Ni = nGridVec(i);           % size of grid in dimension i
        Nother = S(1).nSts/Ni;      % number of cells to add over in all 
                                    % other dimensions
        dimOrder = 1:nDim;
        dimOrder(i) = [];
        dimOrder = [i  dimOrder];   % Permute entire array and bring dimension
        nn = permute(S(1).nStat,dimOrder); % in first place.
        nn = reshape(nn,[Ni Nother]); % collapse all other dimensions into one
                                    % (i.e. the others are vectorized)
        S(i).nMarg = sum(nn,2);     % Obtain marginal density for dimension i.
    end
    out = S;                        % Return the object.
else
    vv = nGridVec;
    SizVec = vv(dimVec);            % Get size vector for the dimensions 
                                    % given in dimVec: [100; 50] for
                                    % example
    Nother = S(1).nSts / prod(SizVec); % number of points to be summed over 
                                    % in the other dimensions.
    dimOrder = 1:nDim;
    dimOrder(dimVec) = [];          % put dimensions in dimVec first in 
    dimOrder = [dimVec  dimOrder];  % order of dimensions, e.g.: [3 4 1 2].
    
    nn = permute(S(1).nStat,dimOrder);
    nn = reshape(nn,[SizVec' Nother]);  % collapse other dimensions to one
    out = sum(nn, length(dimVec)+1);    % and sum over this last one.
end
