function nn = GetStatDensity(S,n0)
% Obtains stationary density for object S.
% 
% Inputs:
% Object S, required fields:
%    S(1).nSts:         number of states in grid (normal, not extended)
%    S(1).K:            nSts x nSts: Hamiltonian matrix, see notes
%    S(1).nGridVec:     nDim x 1: size of (normal) grid
% n0:                   optional, size nGridVec, i.e. nGrid1 x ... x
%                       nGridN. Starting density
%
% Output:
% array with size nGridVec, i.e. nGrid1 x ... x nGridN, contains stationary
% density
%
% Usage:
% S = ValueFctIter(S);  % Get equilibrium policies for system S
% n = GetStatDensity(S); % get stationary density
% S(1).nStat = n;       % put density into object

nSts = S(1).nSts;               
if nargin==1                    % start with uniform guess if n0 not given
   n0 = ones(nSts,1) / nSts; 
else                            % vectorize guess if given
   n0 = n0(:);
end

dt = GetDt(S(1).K,0);           % Get time interval that works
FF = ( speye(nSts) + S(1).K*dt)';% Get forward matrix

nnew = n0;                      % Set up for loop
dist = 1;
i = 0;
epsConv = 10^(-9);              % convergence criterion
maxIter = 10^5;                 % maximal number of iterations
while i<maxIter && dist>epsConv
   nold = nnew;
   i=i+1; 
   nnew = FF * nold;            % forward-iterate density
   dist = mean( abs( nnew - nold ) ); % check convergence
end

if i<maxIter                    % tell user about result
    display(sprintf('Density converged after %1.0f iterations',i));
else
    display(sprintf('NO CONVERGENCE of density, stop forced after %1.0f iterations',maxIter));
end

n = reshape( nnew, S(1).nGridVec'); % return the density

nn = n;
