function LOM = GetUpWindLOM(S,iDim)
% Returns the law of motion in dimension iDim according to upwind principle
% 
% Input:
% S: object for finite-element methods. Must contain fields:
%    aUp:       nGrid1 x ... x nGridN array which  was obtained using the 
%               upward numerical derivative in dimension iDim.
%    aDn:       nGrid1 x ... x nGridN array, btained using downward
%               derivative in dimension iDim
% iDim:         scalar. Dimension for which upward LOM is to be calculated.
%
% Output:
% array of same dimensions as S.a, i.e. 
%
% Called by:
% S(1).PolicyFctHdl  (usually this function should call)
%
% Usage:
% Vk = S(1).V.jac(:,:,1,1);           % read out derivative value function
% cUp  = Vk.^(-1/gamma);              % Get policy looking upward
% S(1).aUp = r*kk + yy - cUp;         % set this as upward LOM
% cDn = [ cup(1,:) ; cup(1:end-1) ];  % get policy looking downward
% S(1).aDn = r*kk + yy - cDn;         % set this as downward LOM
% S(1).a = GetUpWindLOM(S,1);         % return upwind law of motion


aUp    = S(iDim).aUp;           % read out law of motion coming from policies
                                % that are obtained from taking UPward 
                                % derivatives...
aDn    = S(iDim).aDn;           % ...and the ones coming from DowNward 
                                % derivatives.

IncrUp = (aUp>0);               % Is state a heading up when looking up?
IncrDn = (aDn>0);               % And when looking down?

aUpWind = IncrUp.*IncrDn.*aUp + (~IncrUp).*(~IncrDn).*aDn + ...
          IncrUp.*(~IncrDn).*(aUp+aDn)/2;
                                % - Set downward LOM if both say down, 
                                % - set upward LOM if both say up,
                                % - stay (i.e. 0) if upward says down and
                                %   downward says up
                                % - Take average in "convex case": upward
                                %   says up and downward says down

LOM = aUpWind;                  % return upwind LOM
