function Z = interpnGen(cellX,V,cellY)
% Used for changes between grid size and "transfering" a function from one 
% grid to the other: Uses interpn.m, but have to do some extra work to
% allow for singleton dimensions. Uses the 'spline' option to interpolate.
%
% Inputs:
% cellX:    1xN cell array with n1 x n2 x .. x nN grids for N variables:
%           This is the old grid.
% V:        n1 x n2 x .. x nN array: values of function on old grid
% cellY:    1xN cell array with m1 x m2 x .. x mN grids for N variables:
%           the grid points for the new grid.
%
% Output:
% m1 x m2 x .. mN array. Values of function on new grid, extrapolated from
% the old one.
% 
% Calls:
% interpn.m
%
% Called by:
% ValueFctIter2.m

N  = length(cellX);             % number of dimensions
nGridVec = size(cellX{1});      % size of old grid
mGridVec = size(cellY{1});      % size of new grid
m = length(nGridVec);
if m<N                          % put trailing singleton dimensions if they
   nGridVec = [ nGridVec, ones(1,N-m) ];  % got lost.
end
NN = sum( nGridVec>1);          % count non-singleton dimensions
XX = cell(1,NN);                % set up cell arrays for squeezed grid
YY = cell(1,NN);
ii=0;
for i=1:N                       % loop over all dimensions
   if nGridVec(i)>1             % if dimension is non-degenerate:
       ii=ii+1;                 % this is a new dimension for the squeezed grid!
       XX{ii} = squeeze( cellX{i} );    % adopt sqeezed grid for this dim.
       YY{ii} = squeeze( cellY{i} );    % also squeeze for the new grid
   end
end
VV = squeeze(V);                % also squeeze the values now
ZZ = interpn(XX{:},VV,YY{:},'spline');  % can use interpn.m after squeezing

Z = reshape(ZZ,mGridVec);       % reshape to old format
