function SS = ValueFctIter(S)
% Performs value-function iteration on object S.
%
% Input:
% Object S with fields:
% S(1).PolicyFctHdl:    function handle. Calculates policy for the model
%                       at hand.
% S(1).V.val:           Contains guess for the value function.
% S(1).VisualizeVFIFctHdl: function handle. Visualizes progress in
%                       value-function iteration. Optional: only has to be
%                       given if VisualizeVFI=1.
% S(1).options:         optional. Options for algorithm.
%      maxIter:         maximal number of iterations.
%      epsConv:         convergence criterion for value function.
%      ShowEvery:       show progress every ... iterations
%      Howard:          0: classical value-function (vf) iteration (update
%                          policies each step, update vf over dt-horizon.
%                       1: Howard algorithm. For new policies, calculate 
%                          vf if played forever.
%                       2: hybrid. Don't update policies each dt-step, keep
%                          them in updating over tUpdatePol periods.
%      tUpdatePol:      see hybrid algorithm Howard=2.
%      Extend:          0: do not extend grid and extrapolate on top of
%                          continuous dimensions (default)
%                       1: extend grid in continuous dimensions
%      dtVar:           0: dt is fixed, read out from options.dt.
%                       1: dt is variable, calculated with function GetDt
%                          and options.pmmin (if given)
%      pmmin:           for dtVar=1 algorithm: Set probability p_m of
%                       staying on same grid point to this value.
%      VisualizeVFI:    1: Visualize progress in value-function iteration.
% other small fields:   See lines 31-42
%
% Output:
% Object S with updated fields:
% S(1).V.val:           Result for value function
% S(1).FU:              flow utility for players under resulting policies
% S(1).distHist:        history of distances between value-function updates
% S(1).dtHist:          history of time steds between updates
%
% Usage:
% S(1).V.val = Vguess;  % Set value-function guess
% S = ValueFctIter(S);  % Perform value-function iteration
% plot(S(1).distHist)   % plot convergence history


% Read out algorithm parameters (were set by FinElObjSetUp.m to default
% values when object was set up, see there for default values):
epsConv      = S(1).options.epsConv;
maxIter      = S(1).options.maxIter;
dtVar        = S(1).options.dtVar;
dt           = S(1).options.dt;
pmmin        = S(1).options.pmmin;
Howard       = S(1).options.Howard;
tUpdatePol   = S(1).options.tUpdatePol;
Extend       = S(1).options.Extend;
ShowEvery    = S(1).options.ShowEvery;
VisualizeVFI = S(1).options.VisualizeVFI;
Message      = S(1).options.Message;

if Howard==1,  Extend=0;  end       % no extension of grid with Howard algorithm

% Read out important parameters from object
nPlr        = S(1).nPlr;            % number of players,
rho         = S(1).params.rho;      % time discount rate,
nSts        = S(1).nSts;            % number of states for normal grid,
nExtSts     = S(1).nExtSts;         % number of states for extended grid.
nGridVec    = S(1).nGridVec;        % Read out grid size of normal grid
nExtGridVec = S(1).nExtGridVec;     % and for extended grid.
FullInd     = S(1).FullInd;         % Get sets of indices.
FullExtInd  = S(1).FullExtInd;

% Read out function handles:
PolicyFctHdl       = S(1).PolicyFctHdl; % function handle: calculates policies
VisualizeVFIFctHdl = S(1).VisualizeVFIFctHdl; % function handle: visualizes 
                                    % progress iteration.


Kextend = S(1).Kextend;
KK = S(1).Kextrap * Kextend;        % Create operator that extends the 
                                    % normal grid to the extended grid
                                    % (both in  vectorized form) and then
                                    % extrapolates to the uppermost grid
                                    % points. KK is an nExtSts x nSts
                                    % sparse matrix.
i=0;
dist = 1;
distHist = zeros(maxIter,1);        % set up histories to check convergence
dtHist = dt*ones(maxIter,1);

if Extend==0                        % read out initial guess for value fct.:
                                    % Exclude extra grid points for conti-
    VV  = S(1).V.val(FullInd{:},:); % nuous dimensions. 
    Vold= reshape(VV,[nSts nPlr]);  % Vectorized value fct. for each player
                                    % (lenght nSts)
    VarraySize = [nGridVec', nPlr]; % size of value-function array
else
                                    % similar for other algorithms, but 
    VV = S(1).V.val(FullExtInd{:},:); % need extra grid points here 
    Vold = reshape(VV,[nExtSts nPlr]); % vectorized value function is 
    VarraySize = [nExtGridVec', nPlr]; % of length nExtSts here.
end                                 
 
%tic                                            
while dist>epsConv && i<maxIter     % start value-function iteration
    i=i+1;                          % Given the last value-function guess
    
   try                              % Catch errors here:
       S = feval(PolicyFctHdl,S);   % Get the policy/law of motion and
   catch                            % catch here to tell at which iteration it happened
       display(sprintf('ERROR in value function iteration at %1.0f iterations, deviation: %e',i, dist))
       i = maxIter;                    % Want to get out of loop now
       Message = 0;
   end
        
    S = GetHamMatrix(S);            % update the K-matrices
    FUvec = reshape( S(1).FU, [nSts, nPlr]); % read out flow utility for
                                    % all players and stack vectorized flow
                                    % utility by player.
    if Howard == 1                  % True Howard algorithm:
        HH = rho*speye( S(1).nSts ) - S(1).K;
                                    % create matrix K from notes
        Vnew = HH \ FUvec;          % find value fct by solving systems of 
                                    % equations for each player.
        S(1).V.val(FullInd{:},:) = reshape(Vnew,VarraySize);
                                    % write value function into object so
                                    % that other functions can see it.
        dist = mean( abs (Vnew(:)-Vold(:)) );
                                    % check convergence
    
    else                            % Non-Howard algorithms:
        
        if dtVar == 1               % If dt is determined at each step:
            dt = GetDt(S(1).K,pmmin); % get dt with function.
            dtHist(i) = dt;         % record history of dt's
        end
        
        if Howard == 2              % Set iterations for sub-loop:
            subIter = max(1,floor(tUpdatePol/dt)); % iterate policies for  
        else                        % tUpdatePol years if Howard=2.
            subIter = 1;            % If not, "classical" algorithm: Just 
        end                         % one iter'n with the new policy, then
                                    % update again.
        Vint = Vold;                % intermediate value function (vectorized).
        
        if Extend==1                % if we work on extended grid:
            for j=1:subIter         % sub-loop: iterate with current policy 
                VintSml = Kextend' * Vint;
                Vsml = FUvec*dt + (1-rho*dt)*VintSml + S(1).Kext*dt*Vint;
                Vnew = KK * Vsml;   % extend to larger grid
                Vint = Vnew;        % update the intermediate step
            end                     % update value function in object:
            S(1).V.val(FullExtInd{:},:) = reshape(Vnew,VarraySize);   
        else                        % Simpler case: no extension of grid.
           for j=1:subIter
               Vnew = FUvec*dt + (1-rho*dt)*Vint + S(1).K*dt*Vint;
               Vint = Vnew;
           end
           S(1).V.val(FullInd{:},:) = reshape(Vnew,VarraySize);   
        end
            
        dist = mean( abs (Vnew(:)-Vold(:)) ) / dt;
    end                             % check convergence, normalized by dt
    
    if mod(i,ShowEvery)==0 && i<maxIter % Give message when reaching steps
        display(sprintf('Value function: %1.0f iterations done',i))
    end
    
    if VisualizeVFI==1              % If user wants to visualize progress:
        distArray = reshape(Vnew-Vold, VarraySize ); % call function handle
        feval(VisualizeVFIFctHdl,i,S,distArray);     % to function that does that
    end
    
    distHist(i) = dist;             % record convergence history
    Vold = Vnew;                    % update value functions
end
%toc                                % can show time elapsed when testing

if Extend==0                        % For non-extended algorithms
    Vnew = KK * Vnew;               % Still need to extend value function
end                                 % to extended grid.
S(1).V.val(FullExtInd{:},:) = reshape(Vnew,[nExtGridVec', nPlr]);

if Message==1                       % if message wanted:
    if i<maxIter                    % tell user about convergence
        display(sprintf('Value function converged after %1.0f iterations',i) );
    else
        display(sprintf('NO CONVERGENCE of value function. Stop forced after %1.0f iterations, deviation: %e',[maxIter dist])) 
    end
end
S(1).distHist = distHist;           % keep history of distances
S(1).dtHist   = dtHist;             % and time steps

SS = S;                             % return object
