function SS = ValueFctIter2(S)
% Performs value-function iteration for object S using ValueFctIter, but
% does pre-run(s) using (a) coarser grid(s).
%
% Input: 
% Object S with fields like in ValueFctIter.m, plus:
% S(1).Nsm:     vector. Sizes of grid in continuous dimensions for
%               pre-runs, e.g. [30, 60] does two pre-runs with grid sizes
%               30 and 60 and then the actual run with the grid given in
%               the object.
%
% Output:
% Object SS, updated fields see ValueFctIter.m
%
% Usage:
% Same as for ValueFctIter.m
%
% Calls:
% ValueFctIter.m

nPlr = S(1).nPlr;                   % read out fundamental fields
nDim = S(1).nDim;
Nsm = S(1).Nsm;                     % grid sizes for pre-runs
K   = length(Nsm);                  % number of pre-runs

VGuessFctHdl = S(1).VGuessFctHdl;

for k=1:K                           % loop: all pre-runs
    Ssm(1).nPlr = S(1).nPlr;                    % set up object with
    Ssm(1).PolicyFctHdl = S(1).PolicyFctHdl;    % smaller grid, first copy
    Ssm(1).VisualizeVFIFctHdl = S(1).VisualizeVFIFctHdl;
    Ssm(1).params = S(1).params;                % the usual stuff from S
    Ssm(1).options = S(1).options;
    Ssm(1).options.ConvCrit = 10^(-5);
    
    
    for i=1:nDim                                % loop over dimensions:
       Ssm(i).discr = S(i).discr;               % take over if discrete or not
       if S(i).discr==1                         % If discrete, just copy
          Ssm(i).GridVec = S(i).GridVec;
          Ssm(i).h       = S(i).h;
       else                                     % If continuos: Set up 
          xLo = S(i).GridVec(1);                % smaller linear grid.
          xHi = S(i).GridVec(end);
          Ssm(i).GridVec = linspace(xLo,xHi,Nsm(k))';
       end
    end
    Ssm = FinElObjSetUp(Ssm);                   % Set up the object
    FullInd = Ssm(1).FullInd;
    if k==1                                     % 1st pre-run: Call value-
        Ssm(1).V.val(FullInd{:},:) = VGuessFctHdl(Ssm); % function-guess
        Ssm = GetJac(Ssm);                      % handle to get guess, use
        Ssm(1).options.VisualizeVFI=1;          % GetJac.m to extrapolate 
    else                                        % to outer grid points.
        V = 0*Ssm(1).V.val;                     % Next pre-runs: Transfer 
        for i=1:nPlr                            % last guess to current
            OldFullInd = SsmOld(1).FullInd;     % grid
            V(FullInd{:},i) = interpnGen( {SsmOld.xGrid} , ...
                  SsmOld(1).V.val( OldFullInd{:} ,i ) ,  {Ssm.xGrid} );
        end
        Ssm(1).V.val = V;
        Ssm(1).options.VisualizeVFI=1;   
    end
    display(sprintf('Pre-run %1.0f with coarse grid (N=%2.0f):',[k, Nsm(k)]))
    Ssm = ValueFctIter(Ssm);                    % do value-function iteration
    SsmOld = Ssm;                               % for current grid size
end

Vsm = Ssm(1).V.val;             % read out last value function result
FullIndSm = Ssm(1).FullInd;     
FullInd = S(1).FullInd;
V = 0*S(1).V.val ;              % bring that to the actual grid
for i=1:nPlr                    % as new final guess
   V(FullInd{:},i) = interpnGen({Ssm.xGrid},Vsm(FullIndSm{:},i),{S.xGrid});
end
S(1).V.val = V;                 % do the main run of value-function iteration:
S = GetJac(S);                  % First extrapolate value function, then
display('Main run with actual grid:')
S = ValueFctIter(S);            % call value-function iteration the last time.
                                
SS = S;                         % return object
